@@ -125,11 +125,9 @@ typedef struct in_addr ne_inet_addr;
 #define USE_CHECK_IPV6
 #endif
 
-/* "Be Conservative In What You Build". */
-#if defined(HAVE_FCNTL) && defined(O_NONBLOCK) && defined(F_SETFL) \
-    && defined(HAVE_GETSOCKOPT) && defined(SO_ERROR) \
-    && defined(HAVE_SOCKLEN_T) && defined(SOL_SOCKET) \
-    && defined(EINPROGRESS)
+
+// Use always non blocking socket
+#ifndef USE_NONBLOCKING_CONNECT
 #define USE_NONBLOCKING_CONNECT
 #endif
 
@@ -259,7 +257,7 @@ static int seed_ssl_prng(void)
 	return 0;
 
 #if defined(EGD_PATH)
-    NE_DEBUG(NE_DBG_SOCKET, "Seeding PRNG from " EGD_PATH "...\n");
+    NE_DEBUG(NE_DBG_SOCKET, "Seeding PRNG from " EGD_PATH "...");
     if (RAND_egd(EGD_PATH) != -1)
 	return 0;
 #elif defined(ENABLE_EGD)
@@ -268,14 +266,14 @@ static int seed_ssl_prng(void)
 				       "/etc/egd-pool", "/etc/entropy" };
 	size_t n;
 	for (n = 0; n < sizeof(paths) / sizeof(char *); n++) {
-	    NE_DEBUG(NE_DBG_SOCKET, "Seeding PRNG from %s...\n", paths[n]);
+	    NE_DEBUG(NE_DBG_SOCKET, "Seeding PRNG from %s...", paths[n]);
 	    if (RAND_egd(paths[n]) != -1)
 		return 0;
 	}
     }
 #endif /* EGD_PATH */
 
-    NE_DEBUG(NE_DBG_SOCKET, "No entropy source found; could not seed PRNG.\n");
+    NE_DEBUG(NE_DBG_SOCKET, "No entropy source found; could not seed PRNG.");
     return -1;
 }
 #endif /* HAVE_OPENSSL */
@@ -340,7 +338,7 @@ int ne_sock_init(void)
     SOCKSinit("neon");
 #endif
 
-#if defined(HAVE_SIGNAL) && defined(SIGPIPE)
+#if defined(HAVE_SIGNAL)
     (void) signal(SIGPIPE, SIG_IGN);
 #endif
 
@@ -434,7 +432,7 @@ ssize_t ne_sock_read(ne_socket *sock, char *buffer, size_t buflen)
     ssize_t bytes;
 
 #if 0
-    NE_DEBUG(NE_DBG_SOCKET, "buf: at %d, %d avail [%s]\n", 
+    NE_DEBUG(NE_DBG_SOCKET, "buf: at %d, %d avail [%s]", 
 	     sock->bufpos - sock->buffer, sock->bufavail, sock->bufpos);
 #endif
 
@@ -538,7 +536,13 @@ static ssize_t write_raw(ne_socket *sock, const char *data, size_t length)
 #endif
 
     do {
-	ret = send(sock->fd, data, length, 0);
+
+// no signal protection
+#if  (defined HAVE_SIGNAL) && (defined MSG_NOSIGNAL)
+    ret = send(sock->fd, data, length, MSG_NOSIGNAL);
+#else
+    ret = send(sock->fd, data, length, 0);
+#endif
     } while (ret == -1 && NE_ISINTR(ne_errno));
 
     if (ret < 0) {
@@ -688,11 +692,11 @@ static int check_alert(ne_socket *sock, ssize_t ret)
 
     if (ret == GNUTLS_E_WARNING_ALERT_RECEIVED) {
         alert = gnutls_alert_get_name(gnutls_alert_get(sock->ssl));
-        NE_DEBUG(NE_DBG_SOCKET, "TLS warning alert: %s\n", alert);
+        NE_DEBUG(NE_DBG_SOCKET, "TLS warning alert: %s", alert);
         return 0;
     } else if (ret == GNUTLS_E_FATAL_ALERT_RECEIVED) {
         alert = gnutls_alert_get_name(gnutls_alert_get(sock->ssl));
-        NE_DEBUG(NE_DBG_SOCKET, "TLS fatal alert: %s\n", alert);
+        NE_DEBUG(NE_DBG_SOCKET, "TLS fatal alert: %s", alert);
         return -1;
     }
     return ret;
@@ -837,7 +841,10 @@ ssize_t ne_sock_readline(ne_socket *sock, char *buf, size_t buflen)
 {
     char *lf;
     size_t len;
-    
+
+    if(!sock)
+        return NE_SOCK_CLOSED;
+
     if ((lf = memchr(sock->bufpos, '\n', sock->bufavail)) == NULL
 	&& sock->bufavail < RDBUFSIZ) {
 	/* The buffered data does not contain a complete line: move it
@@ -1441,7 +1448,7 @@ int ne_sock_connect(ne_socket *sock,
         setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &flag, sizeof flag);
     }
 #endif
-    
+
     ret = connect_socket(sock, fd, addr, htons(port));
     if (ret == 0)
         sock->fd = fd;
@@ -1675,7 +1682,7 @@ int ne_sock_accept_ssl(ne_socket *sock, ne_ssl_context *ctx)
     }
 
     if (SSL_session_reused(ssl)) {
-        NE_DEBUG(NE_DBG_SSL, "ssl: Server reused session.\n");
+        NE_DEBUG(NE_DBG_SSL, "ssl: Server reused session.");
     }
 #elif defined(HAVE_GNUTLS)
     gnutls_init(&ssl, GNUTLS_SERVER);
