@@ -17,7 +17,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
 */
-
 #ifndef DAVIX_AUTH_HPP
 #define DAVIX_AUTH_HPP
 
@@ -25,8 +24,8 @@
 #include <string>
 #include <vector>
 
-#include <utils/davix_types.hpp>
-#include <auth/davixx509cred.hpp>
+#include "../utils/davix_types.hpp"
+#include "davixx509cred.hpp"
 
 
 #ifndef __DAVIX_INSIDE__
@@ -54,9 +53,23 @@ private:
     SessionInfo();
     SessionInfoInternal* data;
     friend class NEONSession;
-    friend class DavixCopyInternal;
+    friend class DavixDelegation;
 };
 
+
+#ifdef __DAVIX_HAS_STD_FUNCTION
+
+///
+/// std::function for advanced authentification with client cert X509
+///
+/// @param info : Session info, contains information about server requesting the certificate
+/// @param cert : Client side credential to provide
+/// @return 0 if client cert is provider with success, can throw DavixException if error occurs
+typedef std::function<int  (const SessionInfo & info, X509Credential& cert)> authFunctionClientCertX509;
+
+#endif
+
+
 ///
 /// callback for advanced authentification with client cert X509
 /// @param userdata : user defined data
