@@ -21,6 +21,9 @@
 #ifndef DAVFILE_HPP
 #define DAVFILE_HPP
 
+#include <memory>
+#include <istream>
+#include <ostream>
 #include <davixcontext.hpp>
 #include <params/davixrequestparams.hpp>
 #include <file/davix_file_info.hpp>
@@ -43,6 +46,8 @@
 namespace Davix{
 
 
+struct StatInfo;
+
 ///
 /// @class DavFile
 /// @brief Davix File Interface
@@ -52,12 +57,31 @@ class DAVIX_EXPORT DavFile
 {
 public:
     struct DavFileInternal;
+
+
+    class Iterator{
+        friend struct DavFileInternal;
+        public:
+            Iterator() : d_ptr() {}
+            Iterator(const Iterator & orig) : d_ptr(orig.d_ptr){}
+
+            bool next();
+
+            const std::string & name();
+            const StatInfo & info();
+        private:
+           struct Internal;
+
+           std::shared_ptr<Internal> d_ptr;
+    };
+
     ///
     /// \brief default constructor
     /// \param c context
     /// \param url Remote File URL
     ///
     DavFile(Context & c, const Uri & url);
+    DavFile(Context &c, const RequestParams & params, const Uri &url);
     DavFile(const DavFile & orig);
     ///
     /// \brief destructor
@@ -76,7 +100,7 @@ public:
     /// @param params  Davix Request parameters
     /// @param err  DavixError error report
     /// @return  Replica vector, if error is found return empty vector and set err properly
-    std::vector<DavFile> getReplicas(const RequestParams* params, DavixError** err);
+    std::vector<DavFile> getReplicas(const RequestParams* params, DavixError** err) throw();
 
     ///
     ///  @brief Vector read operation
@@ -95,7 +119,7 @@ public:
                           const DavIOVecInput * input_vec,
                           DavIOVecOuput * ioutput_vec,
                           const dav_size_t count_vec,
-                          DavixError** err);
+                          DavixError** err) throw();
 
     ///
     ///  @brief Partial position independant read.
@@ -115,7 +139,7 @@ public:
                             void* buff,
                             dav_size_t count,
                             dav_off_t offset,
-                            DavixError** err);
+                            DavixError** err) throw();
 
 
     ///
@@ -128,7 +152,7 @@ public:
     ///
     dav_ssize_t getToFd(const RequestParams* params,
                             int fd,
-                            DavixError** err);
+                            DavixError** err) throw();
 
     ///
     ///  @brief Get the first 'size_read' bytes of the file and write it to fd
@@ -142,7 +166,7 @@ public:
     dav_ssize_t getToFd(const RequestParams* params,
                             int fd,
                             dav_size_t size_read,
-                            DavixError** err);
+                            DavixError** err) throw();
 
     ///
     ///  @brief Get the full file content in a dynamically allocated buffer
@@ -154,24 +178,77 @@ public:
     ///
     dav_ssize_t getFull(const RequestParams* params,
                             std::vector<char> & buffer,
-                            DavixError** err);
+                            DavixError** err) throw();
+
 
     ///
-    ///  @brief create and replace the file with the content
-    ///     of the file descriptor fd
+    ///  @brief Get the full file content
+    ///
+    ///  @param params Davix request Parameters
+    ///  @param buffer reference to a vector for the result
+    ///  @return total number of bytes read, or -1 if error occures
+    ///
+    /// Get the file content in a dynamically allocated buffer
+    ///
+    ///  WARNING: this operation is without size limit for the content
+    ///
+    dav_ssize_t get(const RequestParams* params,
+                    std::vector<char> & buffer);
+
+
+
+    ///
+    ///  @brief Create/Replace file content
     ///
     ///  @param params Davix request Parameters
     ///  @param fd file descriptor
-    ///  @param size_write number of bytes to write    
-    ///  @param err Davix Error report
-    ///  @return 0 if success, or -1 if error occures
+    ///  @param size_write number of bytes to write
+    ///  @throw DavixException if an error occurs
     ///
-    int putFromFd(const RequestParams* params,
-                  int fd,
-                  dav_size_t size_write,
-                  DavixError** err);
+    ///  Create / Replace the file.
+    ///  Read the new content from the file descriptor fd for a maximum of size_write bytes.
+    void put(const RequestParams* params, int fd, dav_size_t size_write);
 
 
+    ///
+    ///  @brief Create/Replace file content
+    ///
+    ///  @param params Davix request Parameters
+    ///  @param buffer buffer with data to write
+    ///  @param size_write number of bytes to write
+    ///  @throw DavixException if an error occurs
+    ///
+    ///  Set a new content for the file.
+    ///  The new content comes from buffer
+    void put(const RequestParams* params, const char* buffer, dav_size_t size_write);
+
+#ifdef __DAVIX_HAS_STD_FUNCTION
+
+    ///
+    ///  @brief Create/Replace file content
+    ///
+    ///  @param params Davix request Parameters
+    ///  @param callback data provider callback
+    ///  @param size Total size of the data to write
+    ///  @throw DavixException if an error occurs
+    ///
+    ///  Set a new content for the file.
+    ///  The new content comes from a data provider callback
+    void put(const RequestParams* params, const DataProviderFun & callback, dav_size_t size);
+
+#endif
+
+    ///
+    /// @brief move
+    /// @param params Davix request Parameters
+    /// @param destination Destination Resource
+    ///
+    /// Move the current resource to Destination
+    /// The result of the operation depend of the protocol used.
+    /// Protocol supported currently: WebDav
+    ///
+    void move(const RequestParams* params, DavFile & destination);
+
 
     ///
     ///  @brief Suppress the current entity.
@@ -203,22 +280,41 @@ public:
     ///  Exception safe version of @ref makeCollection(const RequestParams *params = NULL)
     int makeCollection(const RequestParams* params,
                        DavixError** err) throw();
+
     ///
-    /// @brief execute a POSIX-like stat() query
+    /// @brief execute a file meta-data query
     ///
     ///  @param params Davix request Parameters
     ///  @param st stat struct
     ///  @param err Davix Error report
     ///  @return 0 if success, or -1 if error occures
     ///
-    int stat(const RequestParams* params, struct stat * st, DavixError** err);
+    StatInfo & statInfo(const RequestParams* params, StatInfo & info);
+
+    ///
+    /// @brief execute a POSIX-like stat() query
+    ///
+    ///  @param params DaviX request parameters
+    ///  @param st stat struct
+    ///  @param err Davix Error report
+    ///  @return 0 if success, or -1 if error occures
+    ///
+    int stat(const RequestParams* params, struct stat * st, DavixError** err) throw();
+
+    ///
+    /// @brief Collection listing
+    ///
+    ///  @param params DaviX request parameters
+    ///  @return Iterator to the collection
+    Iterator  listCollection(const RequestParams* params);
 
 
     ///
     /// @brief compute checksum of the file
     /// with the given algorithm (MD5, CRC32, ADLER32)
     ///
-    ///  Depend of server implementation
+    ///  server implementation dependend
+    ///  Davix::checksum support LCGDM-DAV, dCache Jetty and Aws S3 checksum support
     ///
     /// @param params request parameters
     /// @param checksm checksum buffer
@@ -232,6 +328,7 @@ public:
     /// Exception safe version of @ref checksum
     int checksum(const RequestParams *params, std::string & checksm, const std::string & chk_algo, DavixError **err) throw();
 
+
     ///
     /// @brief provide informations on the next file operation
     ///
@@ -248,60 +345,27 @@ private:
     DavFileInternal* d_ptr;
 
 public:
-    /// @deprecated deprecated, will be removed in 1.0
-    dav_ssize_t getAllReplicas(const RequestParams* params,
-                                    ReplicaVec & vec, DavixError** err);
+    /// @deprecated deprecated, will be removed in 2.0
+    DEPRECATED(dav_ssize_t getAllReplicas(const RequestParams* params,
+                                    ReplicaVec & vec, DavixError** err));
+
+    ///
+    ///  @deprecated please use put() as a replacer, will be removed in 2.0
+    ///
+    DEPRECATED(int putFromFd(const RequestParams* params,
+                  int fd,
+                  dav_size_t size_write,
+                  DavixError** err) throw());
 };
 
 typedef DavFile File;
 
 
-///
-/// @brief The StatInfo struct
-/// @struct Container for File basic metadata
-///
-/// Follow the POSIX stat() structure
-struct StatInfo{
-    StatInfo(): size(0), nlink(0), mode(0), atime(0), mtime(0), ctime(0){
-    }
-
-    /// size in bytes of the resource
-    dav_size_t size;
-    /// number of links to the resource
-    /// optional
-    dav_ssize_t nlink;
-    /// POSIX rights of the resource
-    /// optional, supported with some Webdav servers
-    mode_t mode;
-    /// access time
-    time_t atime;
-    /// modification time
-    time_t mtime;
-    /// creation time
-    time_t ctime;
-
-    /// struct converter from POSIX stat
-    inline void fromPosixStat(const struct stat & st){
-        mode = static_cast<mode_t>(st.st_mode);
-        atime = static_cast<time_t>(st.st_atime);
-        mtime = static_cast<time_t>(st.st_mtime);
-        ctime = static_cast<time_t>(st.st_ctime);
-        size =  static_cast<dav_size_t>(st.st_size);
-        nlink = static_cast<dav_size_t>(st.st_nlink);
-    }
-
-    /// struct converter to POSIX stat
-    inline struct stat & toPosixStat(struct stat & st){
-        st.st_mode = static_cast<mode_t>(mode);
-        st.st_atime = static_cast<time_t>(atime);
-        st.st_mtime = static_cast<time_t>(mtime);
-        st.st_ctime = static_cast<time_t>(ctime);
-        st.st_size =  static_cast<off_t>(size);
-        st.st_nlink = static_cast<nlink_t>(nlink);
-        return st;
-    }
-};
-
 } // Davix
 
+
+// provide stream operator for Davix::File
+std::ostream & operator<<(std::ostream & out, Davix::DavFile & file);
+std::istream & operator>>(std::istream & in, Davix::DavFile & file);
+
 #endif // DAVFILE_HPP
