@@ -17,17 +17,14 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
 */
-
-
-#pragma once
 #ifndef DAVIX_REQUESTPARAMS_HPP
 #define DAVIX_REQUESTPARAMS_HPP
 
 #include <vector>
 #include <string>
 
-#include <utils/davix_uri.hpp>
-#include <auth/davixauth.hpp>
+#include "davix_request_params_types.hpp"
+#include "../auth/davixauth.hpp"
 
 
 /**
@@ -42,49 +39,11 @@
 #endif
 
 
-
 namespace Davix {
 
 
-namespace RequestProtocol{
-    ///
-    /// \brief Http based protocol to use for advanced queries
-    ///
-    enum Protocol{
-        // default
-        Auto=0,
-        // Strict Http without extensions
-        Http,
-        // Use Http + Webdav extension
-        Webdav,
-        // Use Amazon S3 API
-        AwsS3
-    };
-}
-
-namespace MetalinkMode{
-    enum MetalinkMode{
-        // default mode = failover
-        Auto=0,
-        // Disable Metalink Support
-        Disable,
-        // Use only for failover purpose
-        FailOver,
-        // Enable parallel DL
-        XStream
-    };
-}
-
-
 struct RequestParamsInternal;
-///
-/// \brief string for Amazon private key
-///
-typedef std::string AwsSecretKey;
-///
-/// \brief string for Amazon public key
-///
-typedef std::string AwsAccessKey;
+
 
 ///
 /// @class RequestParams
@@ -135,6 +94,15 @@ public:
     const std::pair<std::string,std::string> & getClientLoginPassword() const;
 
 
+#ifdef __DAVIX_HAS_STD_FUNCTION
+    /// set a function for or X509 client side dynamic authentication
+    /// this function overwrite \ref setClientCertCallbackX509
+    void setClientCertFunctionX509(const authFunctionClientCertX509 & callback);
+
+    /// return the function
+    const authFunctionClientCertX509 & getClientCertFunctionX509() const;
+#endif
+
     /// set a callback for X509 client side dynamic authentication
     /// this function overwrite \ref setClientCertX509
     void setClientCertCallbackX509(authCallbackClientCertX509 callback, void* userdata);
@@ -170,7 +138,7 @@ public:
 
     /// define the connexion timeout
     /// conn_timeout is a relative time
-    /// DEFAULT : 180s
+    /// DEFAULT : 30s
     void setConnectionTimeout(struct timespec* conn_timeout);
 
     /// get the current connexion timeout
@@ -196,6 +164,48 @@ public:
     /// return true if the transparent redirection mode is enabled
     bool getTransparentRedirectionSupport() const;
 
+    ///
+    /// \brief number of re-try in case of operation failure
+    /// \param number_retry
+    ///
+    /// define the number of retry attempt  in case of an operation failure
+    void setOperationRetry(int number_retry);
+
+
+    ///
+    /// \brief getOperationRetry
+    /// \return
+    /// get current number of retry attempt, see \ref setOperationRetry for more details
+    int getOperationRetry() const;
+
+
+#ifdef __DAVIX_HAS_STD_FUNCTION
+    ///
+    /// @brief setTransfertMonitorCb
+    /// @param cb
+    ///
+    ///  define a transfer callback
+    ///  The transfer callback is called on a regular based
+    ///  when a data transfer operation is progressing ( put, get, copy )
+    ///
+    ///  The callback is called at least once per transfer for the following operations :
+    ///  - DavFile::put
+    ///  - DavixFile::get / getV
+    ///  - Posix::read / pread / write / prwrite / preadVec
+    ///
+    ///
+    void setTransfertMonitorCb(const TransferMonitorCB & cb);
+
+    ///
+    /// \brief getTransferMonitorCb
+    /// \return current transfer monitor callback
+    ///
+    ///  see \ref setTransfertMonitorCb for more details
+    ///
+    const TransferMonitorCB & getTransferMonitorCb() const;
+
+#endif //__DAVIX_HAS_STD_FUNCTION
+
     /// set the user agent for the associated request
     void setUserAgent(const std::string & user_agent);
 
@@ -214,7 +224,7 @@ public:
     void setMetalinkMode( const MetalinkMode::MetalinkMode mode);
 
     /// get the Current Metalink mode
-    const MetalinkMode::MetalinkMode getMetalinkMode() const;
+    MetalinkMode::MetalinkMode getMetalinkMode() const;
 
     /// set the keep alive value of the associated session
     void setKeepAlive(const bool keep_alive_flag);
