@@ -56,6 +56,7 @@ namespace Davix {
 typedef dav_ssize_t (*HttpBodyProvider)(void *userdata,
                                     char *buffer, dav_size_t buflen);
 
+
 ///////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////
 
@@ -242,9 +243,12 @@ public:
     /// finish a request stated with beginRequest
     int endRequest(DavixError** err);
 
-    /// get a reference to the internal anwser content buffer
+    /// return the body of the answer
     const char* getAnswerContent();
 
+    /// return the body of the answer in a vector
+    std::vector<char>  & getAnswerContentVec();
+
     /// get content length
     /// @return content size, return -1 if chunked
     dav_ssize_t getAnswerSize() const;
@@ -275,10 +279,10 @@ public:
 
 
     /// @deprecated not in use anymore
-    HttpCacheToken* extractCacheToken() const;
+    DEPRECATED(HttpCacheToken* extractCacheToken() const);
 
     /// @deprecated not in use anymore
-    void useCacheToken(const HttpCacheToken* token);
+    DEPRECATED(void useCacheToken(const HttpCacheToken* token));
 
     /// set a HttpRequest flag
     void setFlag(const RequestFlag::RequestFlag flag, bool value);
@@ -364,12 +368,6 @@ public:
     PropfindRequest(Context & context, const Uri & uri, DavixError** err);
 };
 
-
-/// \cond PRIVATE_SYMBOLS
-DAVIX_EXPORT bool httpcodeIsValid(int code);
-
-DAVIX_EXPORT void httpcodeToDavixCode(int code, const std::string & scope, const std::string & end_message, DavixError** err);
-/// \endcond PRIVATE_SYMBOLS
 }
 
 #endif // DAVIX_HTTPREQUEST_H
