@@ -5,6 +5,9 @@
 #include <neon/neonsessionfactory.hpp>
 #include <utils/davix_logger_internal.hpp>
 
+#include <dlfcn.h>
+
+
 namespace Davix {
 
 /// @cond HIDDEN_SYMBOLS
@@ -17,6 +20,20 @@ static NEONSessionFactory & SessionFactoryFromContext(Context & c);
 };
 
 
+// libpath handler
+struct LibPath{
+    LibPath();
+
+    std::string path;
+
+};
+
+
+const std::string & getLibPath();
+
+
+
+
 ///@endcond
 
 } // namespace Davix
