@@ -70,7 +70,7 @@ void HttpIoVecSetupErrorMultiPartSize( DavixError** err, dav_off_t req_offset, d
     DavixError::setupError(err, HttpIoVec_scope(), StatusCode::InvalidServerResponse, ss.str());
 }
 
-inline char* header_delimiter(char* buffer, dav_size_t len, DavixError** err){
+inline char* header_delimiter(char* buffer, dav_size_t len){
     char* p = std::find(buffer, buffer + len, ':');
     return (p < buffer + len)?p:NULL;
 }
@@ -80,8 +80,7 @@ int davIOVecProvider(const DavIOVecInput *input_vec, dav_ssize_t & counter, dav_
     if(counter < number){
         begin = input_vec[counter].diov_offset;
         end = std::max<dav_off_t>(begin + input_vec[counter].diov_size -1, begin);
-        counter ++;
-        return counter;
+        return ++counter;
     }
     return -1;
 }
@@ -100,13 +99,11 @@ dav_ssize_t HttpIOVecOps::preadVec(IOChainContext & iocontext, const DavIOVecInp
             return res;
     }
 
-
-
     DavixError * tmp_err=NULL;
     dav_ssize_t tmp_ret=-1, ret = 0;
     ptrdiff_t p_diff=0;
     dav_ssize_t counter = 0;
-    // determine the Range header request in order to determine the number of request
+    // generator of offset
     boost::function<int (dav_off_t &, dav_off_t &)> offsetProvider( boost::bind(davIOVecProvider, input_vec, counter, (dav_ssize_t) count_vec,
                          _1, _2));
 
@@ -116,10 +113,10 @@ dav_ssize_t HttpIOVecOps::preadVec(IOChainContext & iocontext, const DavIOVecInp
     std::vector< std::pair<dav_size_t, std::string> > vecRanges = generateRangeHeaders(3900, offsetProvider);
 
 
-    DAVIX_DEBUG(" -> getPartialVec operation for %d vectors", count_vec);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, " -> getPartialVec operation for {} vectors", count_vec);
 
     for(std::vector< std::pair<dav_size_t, std::string> >::iterator it = vecRanges.begin(); it < vecRanges.end(); ++it){
-        DAVIX_DEBUG(" -> getPartialVec request for %ld chunks", it->first);
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, " -> getPartialVec request for {} chunks", it->first);
 
         if(it->first == 1){ // one chunk only : no need of multi part
             TRY_DAVIX{
@@ -159,7 +156,7 @@ dav_ssize_t HttpIOVecOps::preadVec(IOChainContext & iocontext, const DavIOVecInp
     }
 
 
-    DAVIX_DEBUG(" <- getPartialVec operation for %d vectors", count_vec);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, " <- getPartialVec operation for {} vectors", count_vec);
     checkDavixError(&tmp_err);
     return ret;
 }
@@ -170,7 +167,7 @@ dav_ssize_t HttpIOVecOps::readPartialBufferVecRequest(HttpRequest & _req,
                           const dav_size_t count_vec, DavixError** err){
     dav_ssize_t ret=-1;
     DavixError* tmp_err=NULL;
-    DAVIX_TRACE(" -> Davix Vector operation");
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, " -> Davix Vector operation");
     if( _req.beginRequest(&tmp_err)  == 0){
         const int retcode = _req.getRequestCode();
         switch(retcode){
@@ -182,12 +179,12 @@ dav_ssize_t HttpIOVecOps::readPartialBufferVecRequest(HttpRequest & _req,
                  ret = simulateMultiPartRequest(_req, input_vec, output_vec, count_vec, &tmp_err);
                  break;
              default:
-                 httpcodeToDavixCode(_req.getRequestCode(),davix_scope_http_request(),", ", &tmp_err);
+                 httpcodeToDavixError(_req.getRequestCode(),davix_scope_http_request(),", ", &tmp_err);
         }
     }
 
     DavixError::propagateError(err, tmp_err);
-    DAVIX_TRACE(" <- Davix Vector operation");
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, " <- Davix Vector operation");
     return ret;
 }
 
@@ -200,7 +197,7 @@ int http_extract_boundary_from_content_type(const std::string & buffer, std::str
         if( tokens.size() >= 1
             && tokens[0].size() > 0
             && tokens[0].size() <= 70){
-            DAVIX_TRACE("Multi part boundary: %s", boundary.c_str());
+            DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Multi part boundary: {}", boundary);
             std::swap(boundary,tokens[0]);
             return 0;
         }
@@ -231,7 +228,7 @@ int get_multi_part_info(const HttpRequest& req, std::string & boundary, DavixErr
 // return 0 -> not a content length header, return -1 : not a header or error, return 1 : success
 int find_header_params(char* buffer, dav_size_t buffer_len, dav_size_t* part_size, dav_off_t* part_offset){
     static const std::string delimiter(" bytes-/\t");
-    char * p = header_delimiter(buffer, buffer_len, NULL);
+    char * p = header_delimiter(buffer, buffer_len);
     if(p == NULL)
         return -1;
     std::string header_type(buffer, p - buffer);
@@ -307,7 +304,7 @@ dav_ssize_t copyChunk(HttpRequest & req, const DavIOVecInput *i,
 
     DavixError* tmp_err=NULL;
     dav_ssize_t ret;
-    DAVIX_DEBUG("Davix::parseMultipartRequest::copyChunk copy %ld bytes with offset %ld", i->diov_size, i->diov_offset);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "Davix::parseMultipartRequest::copyChunk copy {} bytes with offset {}", i->diov_size, i->diov_offset);
     // if size ==0, request set to 1 byte due to server behavior, read the stupid byte and skip
     if( i->diov_size ==0){
         char trash[2];
@@ -324,7 +321,7 @@ dav_ssize_t copyChunk(HttpRequest & req, const DavIOVecInput *i,
     if(tmp_err){
         DavixError::propagateError(err, tmp_err);
     }else{
-        DAVIX_DEBUG("Davix::parseMultipartRequest::copyChunk %ld bytes copied with success",ret);
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "Davix::parseMultipartRequest::copyChunk {} bytes copied with success",ret);
     }
     return ret;
 }
@@ -337,17 +334,17 @@ dav_ssize_t HttpIOVecOps::parseMultipartRequest(HttpRequest & _req,
     std::string boundary;
     dav_ssize_t ret = 0, tmp_ret =0;
     dav_size_t off=0;
-    DAVIX_TRACE("Davix::parseMultipartRequest multi part parsing");
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Davix::parseMultipartRequest multi part parsing");
 
     if(get_multi_part_info(_req, boundary, err)  != 0 ){
-        DAVIX_TRACE("Invalid Header Content info for multi part request");
+        DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Invalid Header Content info for multi part request");
         HttpIoVecSetupErrorMultiPart(err);
         return -1;
     }
-    DAVIX_DEBUG("Davix::parseMultipartRequest multi-part boundary %s", boundary.c_str());
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "Davix::parseMultipartRequest multi-part boundary {}", boundary);
 
     while(off < count_vec){
-       DAVIX_DEBUG("Davix::parseMultipartRequest try to find chunk offset:%ld size %ld", input_vec[off].diov_offset, input_vec[off].diov_size);
+       DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "Davix::parseMultipartRequest try to find chunk offset:{} size {}", input_vec[off].diov_offset, input_vec[off].diov_size);
        ChunkInfo infos;
        int n_try = 0;
        if( (tmp_ret = parse_multi_part_header(_req, boundary, infos,
@@ -356,7 +353,7 @@ dav_ssize_t HttpIOVecOps::parseMultipartRequest(HttpRequest & _req,
        }
 
        if(infos.offset == 0 &&  infos.size == 0 && infos.bounded == true){
-            DAVIX_DEBUG("Davix::parseMultipartRequest multi-part : end of the request found %ld chunks treated on %ld", off, count_vec);
+            DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "Davix::parseMultipartRequest multi-part : end of the request found {} chunks treated on {}", off, count_vec);
             return ret;
        }
 
@@ -371,12 +368,18 @@ dav_ssize_t HttpIOVecOps::parseMultipartRequest(HttpRequest & _req,
 
        if( (tmp_ret = copyChunk(_req, &input_vec[off], &output_vec[off], err)) <0 )
            return -1;
+
        ret += tmp_ret;
-       DAVIX_DEBUG("Davix::parseMultipartRequest chunk parsed with success, next chunk..");
+       DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "Davix::parseMultipartRequest chunk parsed with success, next chunk..");
        off++;
     }
 
-    DAVIX_TRACE("Davix::parseMultipartRequest end %d %d", off, count_vec);
+    // finish with success, dump the remaining part of the query to end the request properly
+    char buffer[255];
+    while( _req.readBlock(buffer, 255, NULL) > 0);
+
+
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Davix::parseMultipartRequest end {} {}", off, count_vec);
     return ret;
 }
 
@@ -391,7 +394,7 @@ bool is_a_start_boundary_part(char* buffer, dav_size_t s_buff, const std::string
             }
         }
     }
-    DAVIX_TRACE("Invalid boundary delimitation");
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Invalid boundary delimitation");
     HttpIoVecSetupErrorMultiPart(err);
     return false;
 }
@@ -460,7 +463,7 @@ static void balance_iterator_windows(MapChunk & m,
 static void fill_concerned_chunk_buffer(MapChunk & m,
                                         MapChunk::iterator & start, MapChunk::iterator & end,
                                         char* buffer, dav_ssize_t read_size, dav_ssize_t pos){
-
+    (void) m;
     for(MapChunk::iterator it = start; it != end; it++){
         const dav_ssize_t size_part = (*it).second._in->diov_size;
         const dav_off_t off_part = (*it).second._in->diov_offset;
@@ -489,7 +492,7 @@ static dav_ssize_t sum_all_chunk_size(const MapChunk & cmap){
 dav_ssize_t HttpIOVecOps::simulateMultiPartRequest(HttpRequest & _req, const DavIOVecInput *input_vec,
                                  DavIOVecOuput * output_vec,
                    const dav_size_t count_vec, DavixError** err){
-    DAVIX_TRACE(" -> Davix vec : 200 full file, simulate vec io");
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, " -> Davix vec : 200 full file, simulate vec io");
     MapChunk cmap;
     dav_ssize_t total_read_size=0, tmp_read_size;
     char buffer[DAVIX_READ_BLOCK_SIZE];
@@ -504,7 +507,7 @@ dav_ssize_t HttpIOVecOps::simulateMultiPartRequest(HttpRequest & _req, const Dav
     if(tmp_read_size < 0)
         return -1;
 
-    DAVIX_TRACE(" <- Davix vec : 200 full file, simulate vec io");
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, " <- Davix vec : 200 full file, simulate vec io");
     return sum_all_chunk_size(cmap);
 }
 
