@@ -58,14 +58,20 @@ private:
 
 
 
-static void neonrequest_eradicate_session(NEONSession& sess, DavixError ** err){
-    if(err && *err && (*err)->getStatus() == StatusCode::ConnectionProblem){
-        DAVIX_DEBUG("Connexion problem: eradicate session.....");
-        sess.disable_session_reuse();
+void configureRequestParamsProto(const Uri &uri, RequestParams &params){
+    if(params.getProtocol() == RequestProtocol::Auto){
+        const std::string & proto = uri.getProtocol();
+        if( proto.compare(0,2,"s3") ==0){
+            params.setProtocol(RequestProtocol::AwsS3);
+        }else if ( proto.compare(0, 3,"dav") ==0){
+            params.setProtocol(RequestProtocol::Webdav);
+        }
     }
+
 }
 
 
+
 void neon_generic_error_mapper(int ne_status, StatusCode::Code & code, std::string & str){
     switch(ne_status){
         case NE_OK:
@@ -107,24 +113,6 @@ void neon_generic_error_mapper(int ne_status, StatusCode::Code & code, std::stri
 }
 
 
-// convert standard neon error to davix code
-void neon_to_davix_code(int ne_status, ne_session* sess, const std::string & scope, DavixError** err){
-    StatusCode::Code code;
-    std::string str;
-    switch(ne_status){
-        case NE_ERROR:
-             str = std::string("Neon error: ").append(ne_get_error(sess));
-             if (str.find("SSL handshake failed") == std::string::npos)
-                 code = StatusCode::ConnectionProblem;
-             else
-                 code = StatusCode::SSLError;
-             break;
-        default:
-            neon_generic_error_mapper(ne_status, code, str);
-    }
-    DavixError::setupError(err,scope, code, str);
-}
-
 // convert neon_simple_request error to davix code,
 void neon_simple_req_code_to_davix_code(int ne_status, ne_session* sess, const std::string & scope, DavixError** err){
     StatusCode::Code code;
@@ -139,7 +127,7 @@ void neon_simple_req_code_to_davix_code(int ne_status, ne_session* sess, const s
              }else{
                  code = StatusCode::ConnectionProblem;
              }
-             str = std::string("Neon error: ").append(str_error);
+             str = std::string("(Neon): ").append(str_error);
              break;
         }
         default:
@@ -158,6 +146,7 @@ NEONRequest::NEONRequest(HttpRequest & h, Context& context, const Uri & uri_req)
     _current( new Uri(uri_req)),
     _orig(_current),
     _number_try(0),
+    _total_read_size(0),
     _last_read(-1),
     _vec(),
     _vec_line(),
@@ -168,6 +157,7 @@ NEONRequest::NEONRequest(HttpRequest & h, Context& context, const Uri & uri_req)
     _fd_content(-1),
     _content_provider(),
     _ans_size(-1),
+    _expiration_time(),
     _request_type("GET"),
     _h(h),
     _c(context),
@@ -175,32 +165,33 @@ NEONRequest::NEONRequest(HttpRequest & h, Context& context, const Uri & uri_req)
     req_running(false),
     _last_request_flag(0),
     _headers_field(){
-
-    if(davix_get_log_level() & DAVIX_LOG_TRACE){
-        ne_debug_init(stderr, NE_DBG_HTTP | NE_DBG_HTTPAUTH | NE_DBG_HTTPPLAIN | NE_DBG_HTTPBODY | NE_DBG_SSL);
-    }
-
 }
 
 
-
-
 NEONRequest::~NEONRequest(){
     // safe destruction of the request
-    reqReset();
+    resetRequest();
+}
+
+void NEONRequest::eradicateSession(){
+   if(_neon_sess.get() != NULL){
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "Connection problem: eradicate session");
+       _neon_sess->disable_session_reuse();
+    }
 }
 
 
-void NEONRequest::reqReset(){
+
+void NEONRequest::resetRequest(){
     if(req_running) {
         endRequest(NULL);
     }
-    free_request();
+    freeRequest();
 }
 
-int NEONRequest::create_req(DavixError** err){
+int NEONRequest::createRequest(DavixError** err){
     if(_req){
-       reqReset();
+       resetRequest();
     }
 
     boost::shared_ptr<Uri> redir_url = ContextExplorer::SessionFactoryFromContext(_c).redirectionResolve(_request_type, *_current);
@@ -208,16 +199,16 @@ int NEONRequest::create_req(DavixError** err){
         _current.swap(redir_url);
     }
 
-    if( pick_sess(err) < 0)
+    if( instanceSession(err) < 0)
         return -1;
 
     _req= ne_request_create(_neon_sess->get_ne_sess(), _request_type.c_str(), _current->getPathAndQuery().c_str());
-    configure_req();
+    configureRequest();
 
     return 0;
 }
 
-int NEONRequest::pick_sess(DavixError** err){
+int NEONRequest::instanceSession(DavixError** err){
     DavixError * tmp_err=NULL;
     _neon_sess.reset(static_cast<NEONSession*>(new NEONSessionExtended(this, *_current, params, &tmp_err)));
     if(tmp_err){
@@ -230,14 +221,36 @@ int NEONRequest::pick_sess(DavixError** err){
 
 ssize_t NEONRequest::neon_body_content_provider(void* userdata, char* buffer, size_t buflen){
 	NEONRequest* req = static_cast<NEONRequest*>(userdata);
-	return (ssize_t) req->_content_provider.callback(req->_content_provider.udata, buffer, buflen);
+     return (ssize_t) req->_content_provider.callback(req->_content_provider.udata, buffer, buflen);
 }
 
-void NEONRequest::configure_req(){
+bool NEONRequest::checkTimeout(DavixError **err){
+    if(_expiration_time.isValid() && _expiration_time < Chrono::Clock(Chrono::Clock::Monolitic).now()){
+        std::ostringstream ss;
+        ss << "timeout of " << params.getOperationTimeout()->tv_sec << "s";
+        DavixError::setupError(err, davix_scope_http_request(), StatusCode::OperationTimeout, ss.str());
+        return true;
+    }
+    return false;
+}
+
+void NEONRequest::configureRequest(){
+    // reconfigure protos
+    configureRequestParamsProto(*_current, params);
+
+    std::copy(params.getHeaders().begin(), params.getHeaders().end(), std::back_inserter(_headers_field));
+
     // configure S3 params if needed
     if(params.getProtocol() == RequestProtocol::AwsS3)
         configureS3params();
 
+    // setup timeout
+    if(_expiration_time.isValid() == false
+        && params.getOperationTimeout()->tv_sec != 0){
+        using namespace Chrono;
+        _expiration_time = Clock(Clock::Monolitic).now() + Duration(params.getOperationTimeout()->tv_sec);
+    }
+
     // setup headers
     for(size_t i=0; i< _headers_field.size(); ++i){
         ne_add_request_header(_req, _headers_field[i].first.c_str(),  _headers_field[i].second.c_str());
@@ -256,45 +269,15 @@ void NEONRequest::configure_req(){
         ne_set_request_body_provider(_req, _content_len,
                                      &neon_body_content_provider, this);
     }else if(_content_ptr && _content_len >0){
-        ne_set_request_body_buffer(_req, _content_ptr, _content_len);       
+        ne_set_request_body_buffer(_req, _content_ptr, _content_len);
     }
-
-    const HeaderVec &  h  = params.getHeaders();
-    for (HeaderVec::const_iterator i = h.begin();
-         i != h.end(); i++) {
-        ne_add_request_header(_req, i->first.c_str(), i->second.c_str());
-      }
 }
 
 
-static std::string extract_bucket(const Uri & uri){
-    const std::string & hostname = uri.getHost();
-    std::string::const_iterator it = std::find(hostname.begin(), hostname.end(),'.');
-    return std::string(hostname.begin(), it);
-}
-
 void NEONRequest::configureS3params(){
-    struct tm utc_current;
-    time_t t = time(NULL);
-    char date[255];
-    std::ostringstream ss;
-    date[254]= '\0';
-#ifdef HAVE_GMTIME_R	
-    gmtime_r(&t, &utc_current);
-#else
-	struct tm* p_utc = gmtime(&t);
-	memcpy(&utc_current, p_utc, sizeof(struct tm));
-#endif	
-    strftime(date, 254, "%a, %d %b %Y %H:%M:%S %z", &utc_current);
-    // add Date header
-    addHeaderField("Date", date);
-    // construct Request token
-    ss << _request_type << "\n"
-       << "\n"          // TODO : implement Content-type and md5 parser
-       << "\n"
-       << date << "\n"
-       << '/' << extract_bucket(*_current)  << _current->getPath();
-    addHeaderField("Authorization", getAwsAuthorizationField(ss.str(), params.getAwsAutorizationKeys().first, params.getAwsAutorizationKeys().second));
+    HeaderVec vec = _headers_field;
+    S3::signRequest(params, _request_type, *_current, vec);
+    vec.swap(_headers_field);
 }
 
 int NEONRequest::processRedirection(int neonCode, DavixError **err){
@@ -304,13 +287,13 @@ int NEONRequest::processRedirection(int neonCode, DavixError **err){
                 && neonCode != NE_RETRY
                 && neonCode != NE_REDIRECT){
             req_started= req_running = false;
-            neon_to_davix_code(neonCode, _neon_sess->get_ne_sess(), davix_scope_http_request(),err);
+            createError(neonCode, err);
             return -1;
         }
         ne_discard_response(_req);              // Get a valid redirection, drop request content
         _last_read =0;
         end_status = ne_end_request(_req);      // submit the redirection
-        if(redirect_request(err) <0){           // accept redirection
+        if(redirectRequest(err) <0){           // accept redirection
             return -1;
         }
         end_status = NE_RETRY;
@@ -322,46 +305,61 @@ int NEONRequest::processRedirection(int neonCode, DavixError **err){
 }
 
 
-int NEONRequest::startRequest(DavixError **err){        
-    if( create_req(err) < 0)
+int NEONRequest::startRequest(DavixError **err){
+    if( createRequest(err) < 0)
             return -1;
-    return negotiate_request(err);
+    return negotiateRequest(err);
 }
 
-int NEONRequest::negotiate_request(DavixError** err){
+int NEONRequest::negotiateRequest(DavixError** err){
 
-    const int n_limit = 10;
+    const int auth_retry_limit = 5;
     int code, status, end_status = NE_RETRY;
     _last_read = -1;
+    _total_read_size = 0;
+
+
+    DAVIX_SCOPE_TRACE(DAVIX_LOG_HTTP, negoreq);
+
+    // check timeout
+    if(checkTimeout(err) == true)
+        return -1;
 
-    DAVIX_DEBUG(" ->   Davix negociate request ... ");
     if(req_started){
         DavixError::setupError(err, davix_scope_http_request(), StatusCode::AlreadyRunning, "Http request already started, Error");
-        DAVIX_DEBUG(" Davix negociate request ... <-");
         return -1;
     }
 
     req_started = req_running = true;
 
-    while(end_status == NE_RETRY && _number_try < n_limit){
-        DAVIX_DEBUG(" ->   NEON start internal request ... ");
+    while(end_status == NE_RETRY && _number_try < auth_retry_limit){
+        DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "NEON start internal request");
 
         if( (status = ne_begin_request(_req)) != NE_OK && status != NE_REDIRECT){
-            if( status == NE_ERROR ){ // bugfix against neon keepalive problem, protection against buggy servers
-                if(strstr(ne_get_error(_neon_sess->get_ne_sess()), "Could not") != NULL
-                   || strstr(ne_get_error(_neon_sess->get_ne_sess()), "Connection reset by peer") != NULL){
-                   DAVIX_DEBUG("KeepAlive Server problem detected, retry");
-                   _number_try++;
-                   redirect_cleanup();
-                   return startRequest(err);
-                }
+            _last_read = -1;
+
+            if( status == NE_ERROR
+                &&  requestCleanup()
+                && (
+                    strstr(ne_get_error(_neon_sess->get_ne_sess()), "Could not") != NULL
+                    || strstr(ne_get_error(_neon_sess->get_ne_sess()), "Connection reset by peer") != NULL
+                    )){
+               DAVIX_SLOG(DAVIX_LOG_VERBOSE, DAVIX_LOG_HTTP, "Connection problem, retry");
+               _number_try++;
+                return startRequest(err);
+            }
+
+            if( status == NE_CONNECT &&  requestCleanup() ){
+                DAVIX_SLOG(DAVIX_LOG_VERBOSE, DAVIX_LOG_HTTP, "Impossible to connect to {}, retry from origin {}", _current->getString(), _orig->getString());
+                _number_try++;
+                return startRequest(err);
             }
 
             req_started= req_running == false;
-            neon_to_davix_code(status, _neon_sess->get_ne_sess(), davix_scope_http_request(),err);
-            neonrequest_eradicate_session(*_neon_sess, err);
+            createError(status, err);
+
+            eradicateSession();
             ContextExplorer::SessionFactoryFromContext(_c).redirectionClean(_request_type, *_orig);
-            DAVIX_DEBUG(" Davix negociate request ... <-");
             return -1;
         }
 
@@ -373,47 +371,47 @@ int NEONRequest::negotiate_request(DavixError** err){
             case 303:
             case 307:
                 if( (end_status = processRedirection(status, err)) <0){
-                   DAVIX_DEBUG(" Davix negociate request ... <-");
                    return -1;
                 }
                 break;
             case 401: // authentification requested, do retry
                 ne_discard_response(_req);
-                _last_read =0;
                 end_status = ne_end_request(_req);
+                _last_read = -1;
 
                 if( end_status != NE_RETRY){
                     req_started= req_running = false;
                     clearAnswerContent();
-                    if(end_status == NE_OK){
-                            DavixError::setupError(err,davix_scope_http_request(),
-                                                   StatusCode::AuthentificationError, "401 Unauthorized Error");
-                    }else{
-                        neon_to_davix_code(status, _neon_sess->get_ne_sess(), davix_scope_http_request(),err);
-                    }
+
                     _number_try++;
-                    if (redirect_cleanup()){
+                    if (requestCleanup()){
                         DavixError::clearError(err);
                         endRequest(NULL);
                         return startRequest(err);
                     }
 
+                    if(end_status == NE_OK){
+                            DavixError::setupError(err,davix_scope_http_request(),
+                                                   StatusCode::AuthentificationError, "401 Unauthorized");
+                            return -1;
+                    }
+
+                    createError(end_status, err);
                     return -1;
                 }
-                DAVIX_DEBUG(" ->   NEON receive %d code, %d .... request again ... ", code, end_status);
+                DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "(NEON) {} code -> request again ", code);
                 break;
-            // dCache token redirection
+            case 403:
             case 501:
-            // cleanup redirection
-            _number_try++;
-            if( redirect_cleanup()){
-                // recursive call, restart request
-                endRequest(NULL);
-                return startRequest(err);
-            }
-
-            return -1;
-
+                 // cleanup redirection
+                _number_try++;
+                if( requestCleanup()){
+                    // recursive call, restart request
+                    endRequest(NULL);
+                    return startRequest(err);
+                }
+		end_status = 0;
+                break;
             default:
                 end_status = 0;
                 break;
@@ -422,32 +420,42 @@ int NEONRequest::negotiate_request(DavixError** err){
         _number_try++;
     }
 
-    if(_number_try >= n_limit){
-        DavixError::setupError(err,davix_scope_http_request(),StatusCode::AuthentificationError,
-                               "Maximum number of retrial reached.");
+    if(_number_try >= auth_retry_limit){
+        if(_neon_sess.get() != NULL && _neon_sess->getLastError(err)){
+            DavixError::setupError(err,davix_scope_http_request(), StatusCode::AuthenticationError,
+                               "Authentication Error, reach maximum number of attempts");
+        }
         return -2;
     }
-    DAVIX_DEBUG(" Davix negociate request ... <-");
     return 0;
 }
 
 
-bool NEONRequest::redirect_cleanup(){
+// mark the session as dirty for destruction, clean any cached redirection due to error
+// detect if the request comes from a cached session or have been redirected
+// if it is the case, return true
+//
+bool NEONRequest::requestCleanup(){
     // cleanup redirection
     ContextExplorer::SessionFactoryFromContext(_c).redirectionClean(_request_type, *_orig);
+
     // disable recycling
     // server supporting broken pipelining will trigger if reused
     _neon_sess->disable_session_reuse();
-    // cancel redirect, maybe outdated ? retry
-    if(_current != _orig){
-        DAVIX_DEBUG(" ->  Auth problem after redirect: cancel redirect and try again");
+
+    // check if we had a redirection
+    // if it's the case redirection can be expired, then
+    if(_current != _orig || _neon_sess->isRecycledSession()){
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, " ->  Error when using reycling of session/redirect : cancel and try again");
+        // disable session reuse for this request, avoid picking up a session already expired
+        params.setKeepAlive(false);
         _current = _orig;
         return true;
     }
     return false;
 }
 
-int NEONRequest::redirect_request(DavixError **err){
+int NEONRequest::redirectRequest(DavixError **err){
     boost::shared_ptr<Uri> old_uri;
     const ne_uri * new_uri = ne_redirect_location(_neon_sess->get_ne_sess());
     if(!new_uri){
@@ -456,7 +464,7 @@ int NEONRequest::redirect_request(DavixError **err){
     }
 
     char* dst_uri = ne_uri_unparse(new_uri);
-    DAVIX_DEBUG("redirection from %s to %s", _current->getString().c_str(), dst_uri);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "redirection from {} to {}", _current->getString(), dst_uri);
 
     // setup new path & session target
     old_uri = _current;
@@ -466,13 +474,13 @@ int NEONRequest::redirect_request(DavixError **err){
 
 
     // recycle old request and session
-    free_request();
+    freeRequest();
     _neon_sess.reset(NULL);
 
     // renew request
     req_started = false;
     // create a new couple of session + request
-    if( create_req(err) <0){
+    if( createRequest(err) <0){
         return -1;
     }
     req_started= true;
@@ -484,7 +492,8 @@ int NEONRequest::executeRequest(DavixError** err){
     _last_request_flag =0;
     _vec.clear();
 
-    DAVIX_DEBUG(" -> NEON start synchronous  request... ");
+    DAVIX_SCOPE_TRACE(DAVIX_LOG_HTTP, execReq);
+
     if( startRequest(err) < 0){
         return -1;
     }
@@ -493,7 +502,7 @@ int NEONRequest::executeRequest(DavixError** err){
         _vec.reserve(std::min<size_t>(getAnswerSize(), 4194304));
 
     while(read_status > 0){
-        DAVIX_DEBUG(" -> NEON Read data flow... ");
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "NEON Read data flow");
         size_t s = _vec.size();
         _vec.resize(s + NEON_BUFFER_SIZE);
         read_status= readBlock(&(_vec[s]), NEON_BUFFER_SIZE, err);
@@ -507,8 +516,9 @@ int NEONRequest::executeRequest(DavixError** err){
     }
 
     if(read_status < 0){
-        if(err && *err == NULL)
-            neon_to_davix_code(read_status, _neon_sess->get_ne_sess(), davix_scope_http_request(), err);
+        if(err && *err == NULL){
+            createError(read_status, err);
+        }
         return -1;
     }
     _vec.push_back('\0');
@@ -520,7 +530,6 @@ int NEONRequest::executeRequest(DavixError** err){
    if( endRequest(err) < 0){
        return -1;
    }
-    DAVIX_DEBUG(" -> End synchronous request ... ");
     _last_request_flag =1; // 1 -> syn request
     return 0;
 }
@@ -546,12 +555,16 @@ dav_ssize_t NEONRequest::readBlock(char* buffer, dav_size_t max_size, DavixError
     if(max_size ==0)
         return 0;
 
+    // check timeout
+    if(checkTimeout(err) == true)
+        return -1;
+
     // take from line buffer
     if(_vec_line.size() > 0){
        if( _vec_line.size() >= max_size){
         std::copy(_vec_line.begin(), _vec_line.begin() + max_size, buffer);
         _vec_line.erase(_vec_line.begin(), _vec_line.begin() + max_size);
-        DAVIX_TRACE("NEONRequest::readBlock read %ld bytes (from buffer)", max_size);
+        DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "NEONRequest::readBlock read {} bytes (from buffer)", max_size);
         return max_size;
        }else{
            const dav_ssize_t n_bytes = _vec_line.size();
@@ -559,13 +572,14 @@ dav_ssize_t NEONRequest::readBlock(char* buffer, dav_size_t max_size, DavixError
            _vec_line.clear();
            read_status = readBlock(buffer + n_bytes, max_size -n_bytes, err);
            const dav_ssize_t ret_value = (read_status >= 0)?(read_status+n_bytes):(-1);
-           DAVIX_TRACE("NEONRequest::readBlock read %ld bytes(from partially)", ret_value);
+           DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "NEONRequest::readBlock read {} bytes(from partially)", ret_value);
            return ret_value;
        }
     }
 
-    if(_last_read ==0)
+    if(_last_read ==0){
         return 0;
+    }
 
     _last_read= read_status= ne_read_response_block(_req, buffer, max_size );
     if(read_status <0){
@@ -573,7 +587,14 @@ dav_ssize_t NEONRequest::readBlock(char* buffer, dav_size_t max_size, DavixError
        req_running = false;
        return -1;
     }
-    DAVIX_TRACE("NEONRequest::readBlock read %ld bytes", read_status);
+
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "NEONRequest::readBlock read {} bytes", read_status);
+
+    _total_read_size += read_status;
+    if(params.getTransferMonitorCb()){
+        dav_ssize_t final_size = getAnswerSize();
+        params.getTransferMonitorCb()(*_current, Transfer::Read, _total_read_size, ((final_size> 0)?(final_size):(0)));
+    }
     return read_status;
 }
 
@@ -611,7 +632,7 @@ dav_ssize_t NEONRequest::readToFd(int fd, dav_size_t read_size, DavixError** err
            }while(write_len >0);
     }
 
-    return (ret> 0)?total:ret;
+    return (ret>= 0)?total:ret;
 }
 
 
@@ -656,7 +677,7 @@ dav_ssize_t NEONRequest::readSegment(char* p_buff, dav_size_t size_read,  DavixE
     dav_ssize_t ret, tmp_ret;
     dav_size_t s_read= size_read;
     ret = tmp_ret = 0;
-    DAVIX_TRACE("Davix::Request::readSegment: want to read %lld bytes ", size_read);
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "Davix::Request::readSegment: want to read {} bytes ", size_read);
 
     do{
         tmp_ret= readBlock(p_buff, s_read, &tmp_err);
@@ -679,19 +700,25 @@ dav_ssize_t NEONRequest::readSegment(char* p_buff, dav_size_t size_read,  DavixE
 
 int NEONRequest::endRequest(DavixError** err){
     int status;
+    (void) err;
+
     if(_req  && req_running == true){
-        req_running = false;
-        if(_last_read > 0){ // if read content, discard it
-            DAVIX_TRACE("Discard remaining content....");
-            ne_discard_response(_req);
-            _last_read =0;
+
+        if(_last_read != 0){ // if read content, discard it
+            DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "(EndRequest)(Libneon) Operation incomplete, kill the connection");
+            ne_abort_request(_req);
+            eradicateSession();
+            _last_read = -1;
+
         }
-        if( (status = ne_end_request(_req)) != NE_OK){
+        status = ne_end_request(_req);
+        if(status != NE_OK && status != NE_REDIRECT) {
             DavixError* tmp_err=NULL;
-            if(_neon_sess.get() != NULL)
-                neon_to_davix_code(status, _neon_sess->get_ne_sess(), davix_scope_http_request(), &tmp_err);
-            if(tmp_err)
-                DAVIX_DEBUG("NEONRequest::endRequest -> error %d Error closing request -> %s ", tmp_err->getStatus(), tmp_err->getErrMsg().c_str());
+            createError(status, err);
+            if(tmp_err){
+                DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "(EndRequest)(Libneon) Suppress broken connection {}  -> {} ", tmp_err->getStatus(), tmp_err->getErrMsg());
+                eradicateSession();
+            }
             DavixError::clearError(&tmp_err);
         }
     }
@@ -715,15 +742,24 @@ const char* NEONRequest::getAnswerContent(){
     return NULL;
 }
 
+
+std::vector<char> & NEONRequest::getAnswerContentVec(){
+    return _vec;
+}
+
 dav_ssize_t NEONRequest::getAnswerSizeFromHeaders() const{
     std::string str_file_size;
     long size=-1;
     if( getAnswerHeader(ans_header_content_length, str_file_size)){
-        size =  strtol(str_file_size.c_str(), NULL, 10);
+        StrUtil::trim(str_file_size);
+        try{
+            size = toType<long, std::string>()(str_file_size);
+        }catch(...){
+            size = -1;
+        }
     }
-    if( size == -1 || size ==  LONG_MAX){
-       DAVIX_TRACE("Bad server answer: %s Invalid, impossible to determine answer size", ans_header_content_length.c_str());
-       size = -1;
+    if( size == -1){
+       DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "Bad server answer: {} Invalid, impossible to determine answer size", ans_header_content_length);
     }
     return static_cast<dav_ssize_t>(size);
 }
@@ -761,7 +797,7 @@ size_t NEONRequest::getAnswerHeaders( HeaderVec & vec_headers) const{
 
 
 void NEONRequest::setRequestBody(const std::string & body){
-    DAVIX_DEBUG("NEONRequest : add request content of size %s ", body.c_str());
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "NEONRequest : add request content of size {} ", body.size());
     _content_body = std::string(body);
     _content_ptr = (char*) _content_body.c_str();
     _content_len = strlen(_content_ptr);
@@ -789,15 +825,51 @@ void NEONRequest::setRequestBody(HttpBodyProvider provider, dav_size_t len, void
 }
 
 
-void NEONRequest::free_request(){
+void NEONRequest::freeRequest(){
     if(_req != NULL){
         ne_request_destroy(_req);
         _req=NULL;
     }
 }
 
+
+void NEONRequest::createError(int ne_status, DavixError **err){
+    StatusCode::Code code;
+    std::string str;
+    switch(ne_status){
+        case NE_ERROR:
+            {
+             const char * neon_error = ne_get_error(_neon_sess->get_ne_sess());
+             str = std::string("(Neon): ").append((neon_error)?(neon_error):"");
+             if (str.find("SSL handshake failed") == std::string::npos)
+                 code = StatusCode::ConnectionProblem;
+             else
+                 code = StatusCode::SSLError;
+            }
+             break;
+        case NE_TIMEOUT:
+            {
+            // check if redirection occured, if redirection occured
+            // report TimeoutRedirectionError, to allow error recovery
+            if(_current != _orig){
+                code = StatusCode::TimeoutRedirectionError;
+                str= "Connection Timeout during redirection on ";
+                str+= _current->getString();
+                break;
+            }
+            neon_generic_error_mapper(ne_status, code, str);
+            }
+            break;
+
+        default:
+            neon_generic_error_mapper(ne_status, code, str);
+    }
+    DavixError::setupError(err, davix_scope_http_request(), code, str);
+}
+
 void NEONRequest::neon_hook_pre_send(ne_request *r, void *userdata,
                    ne_buffer *header){
+    (void) r;
     NEONRequest* req = (NEONRequest*) userdata;
     RequestPreSendHook hook = req->_c.getHook<RequestPreSendHook>();
     if(hook){
@@ -808,6 +880,7 @@ void NEONRequest::neon_hook_pre_send(ne_request *r, void *userdata,
 
 void NEONRequest::neon_hook_pre_rec(ne_request *r, void *userdata,
                                     const ne_status *status){
+    (void) r;
     NEONRequest* req = (NEONRequest*) userdata;
     RequestPreReceHook hook = req->_c.getHook<RequestPreReceHook>();
     if(hook){
