@@ -26,11 +26,11 @@
 #include <memory>
 #include <string>
 
+#include <davix_internal.hpp>
+
 #include <ne_request.h>
 #include <ne_auth.h>
 #include <neon/neonsessionfactory.hpp>
-#include <boost/scoped_ptr.hpp>
-#include <boost/shared_ptr.hpp>
  
 #include <request/httprequest.hpp>
 #include <neon/neonsession.hpp>
@@ -118,6 +118,9 @@ public:
       get a reference to the current result for synchronous full request
      */
     const char* getAnswerContent();
+
+    std::vector<char> & getAnswerContentVec();
+
     /**
      * get content length
      **/
@@ -145,7 +148,7 @@ private:
     // request parameters
     RequestParams params;
     // neon internal field
-    boost::scoped_ptr<NEONSession> _neon_sess;
+    Ptr::Scoped<NEONSession> _neon_sess;
     // request options flag
 
 
@@ -153,9 +156,9 @@ private:
     ne_request * _req;
     boost::shared_ptr<Uri>  _current, _orig;
     // req info
-    size_t _number_try;
+    int _number_try;
     // read info
-    dav_ssize_t _last_read;
+    dav_ssize_t _total_read_size, _last_read;
 
     std::vector<char> _vec;
     std::vector<char> _vec_line;
@@ -170,6 +173,10 @@ private:
 
     // answer length
     mutable dav_ssize_t _ans_size;
+
+    // timeout management
+    Chrono::TimePoint _expiration_time;
+
     // Request string
     std::string _request_type;
     HttpRequest & _h;
@@ -187,27 +194,35 @@ private:
 
     int processRedirection(int neonCode, DavixError** err); // analyze and process redirection if needed
 
-    void reqReset();
+    void resetRequest();
 
-    int pick_sess(DavixError** err);
+    int instanceSession(DavixError** err);
 
-    void configure_req();
+    bool checkTimeout(DavixError** err);
+
+    void configureRequest();
 
     void configureS3params();
 
+    void eradicateSession();
+
     // create initial neon request object
-    int create_req(DavixError** err);
+    int createRequest(DavixError** err);
 
     // negociate the request : authentification, redirection, name resolution
-    int negotiate_request(DavixError** err);
+    int negotiateRequest(DavixError** err);
 
     // redirection logic
-    int redirect_request(DavixError** err);
+    int redirectRequest(DavixError** err);
 
     // redirecttion caching cleaning function
-    bool redirect_cleanup();
+    bool requestCleanup();
+
+    void freeRequest();
+
+
+    void createError(int ne_status, DavixError** err);
 
-    void free_request();
     /**
       internal, try to authentification with pkcs12 credential
     */
@@ -224,6 +239,7 @@ private:
     static void neon_hook_pre_rec(ne_request *req, void *userdata,
                                         const ne_status *status);
 
+
     friend class HttpRequest;
     friend class NEONSessionExtended;
 };
@@ -237,6 +253,8 @@ void neon_to_davix_code(int ne_status,ne_session* sess, const std::string & scop
 
 void neon_simple_req_code_to_davix_code(int ne_status, ne_session* sess, const std::string & scope, DavixError** err);
 
+void configureRequestParamsProto(const Uri &uri, RequestParams &params);
+
 
 } // namespace Davix
 
