@@ -44,6 +44,9 @@ static std::pair<std::string, std::string> userInfo_to_auth(const std::string &
 
 static int validate_all_certificate(void *userdata, int failures,
                                 const ne_ssl_certificate *cert){
+    (void) userdata;
+    (void) failures;
+    (void) cert;
     return 0;
 }
 
@@ -53,39 +56,40 @@ const int n_max_auth = 20;
 void NEONSession::authNeonCliCertMapper(void *userdata, ne_session *sess,
                                          const ne_ssl_dname *const *dnames,
                                          int dncount){
-
+    (void) dncount;
+    (void) sess;
+    (void) dnames;
     NEONSession* req = static_cast<NEONSession*>(userdata);
-    DavixError* tmp_err=NULL;
 
     X509Credential cert;
-    std::pair<authCallbackClientCertX509,void*> retcallback = req->_params.getClientCertCallbackX509();
-    DAVIX_DEBUG("NEONSession > clicert callback ");
-    if( retcallback.first != NULL){
-        DAVIX_DEBUG("NEONSession > call client cert callback ");
+    const RequestParams* params = &req->_params;
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "clicert callback ");
+    DavixError::clearError(&(req->_last_error));
+
+    if(params->getClientCertFunctionX509()){
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "call client cert callback ");
         SessionInfo infos;
-
-        if( retcallback.first(retcallback.second, infos, &cert, &tmp_err) != 0 || cert.hasCert() == false){
-            if(!tmp_err)
-                DavixError::setupError(&tmp_err, davix_scope_http_request(), StatusCode::AuthentificationError,
-                                       "No valid credential given ");
-             DavixError::propagateError(&(req->_last_error), tmp_err);
-             return;
-        }
-
-        ne_ssl_set_clicert(req->_sess, X509CredentialExtra::extract_ne_ssl_clicert(cert));
-        DAVIX_DEBUG("NEONSession > end call client cert callback");
+        TRY_DAVIX{
+            params->getClientCertFunctionX509()(infos, cert);
+            if(cert.hasCert() == false){
+                throw DavixException(davix_scope_x509cred(), StatusCode::AuthentificationError,
+                                     "No valid credential given ");
+            }
+            ne_ssl_set_clicert(req->_sess, X509CredentialExtra::extract_ne_ssl_clicert(cert));
+        }CATCH_DAVIX(&(req->_last_error));
     }
     return;
 }
 
 int NEONSession::provide_login_passwd_fn(void *userdata, const char *realm, int attempt,
                                 char *username, char *password){
+    (void) realm;
     NEONSession * req = static_cast<NEONSession*>(userdata);
     DavixError * tmp_err=NULL;
     int ret =-1;
 
 
-    DAVIX_DEBUG("NEONSession > Try to get auth/password authentification from client");
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "Try to get auth/password authentification from client");
 
      if(attempt > n_max_auth ){
          DavixError::setupError(&(req->_last_error), davix_scope_http_request(), StatusCode::LoginPasswordError,
@@ -105,9 +109,10 @@ int NEONSession::provide_login_passwd_fn(void *userdata, const char *realm, int
               copy_std_string_to_buff(username, NE_ABUFSIZ, id.first.c_str());
               copy_std_string_to_buff(password, NE_ABUFSIZ, id.second.c_str());
      }else if(retcallback.first != NULL){
-         DAVIX_DEBUG("NEONSession > Try callback for login/passwd for %d time", attempt+1);
+         DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "Try callback for login/passwd for {} time", attempt+1);
          SessionInfo infos;
          std::string tmp_login, tmp_password;
+         DavixError::clearError(&(req->_last_error));
          if( (ret = retcallback.first(retcallback.second, infos, tmp_login, tmp_password, attempt, &tmp_err) ) <0){
              if(!tmp_err)
                  DavixError::setupError(&tmp_err, davix_scope_http_request(), StatusCode::LoginPasswordError,
@@ -119,7 +124,7 @@ int NEONSession::provide_login_passwd_fn(void *userdata, const char *realm, int
          copy_std_string_to_buff(password, NE_ABUFSIZ, tmp_password.c_str());
      }
 
-    DAVIX_DEBUG("NEONSession > get login/password with success...try server submission ");
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "get login/password with success...try server submission ");
     return 0;
 
 }
@@ -131,12 +136,13 @@ NEONSession::NEONSession(Context & c, const Uri & uri, const RequestParams & p,
     _sess(NULL),
     _params(p),
     _last_error(NULL),
-    _session_recycling(_f.getSessionCaching()),
+    _session_recycling(_f.getSessionCaching() && p.getKeepAlive()),
+    reused(false),
     _u(uri)
 {
         _f.createNeonSession(p, uri, &_sess, err);
         if(_sess)
-            configureSession(_sess, _u, p, &NEONSession::provide_login_passwd_fn, this, &NEONSession::authNeonCliCertMapper, this);
+            configureSession(_sess, _u, p, &NEONSession::provide_login_passwd_fn, this, &NEONSession::authNeonCliCertMapper, this, reused);
 }
 
 
@@ -144,22 +150,26 @@ NEONSession::~NEONSession(){
 #   ifndef _DISABLE_SESSION_REUSE
         if(_sess){
             if(_session_recycling)
-                _f.storeNeonSession(_sess, NULL);
+                _f.storeNeonSession(_sess);
             else
                 ne_session_destroy(_sess);
         }
 #   endif
-
+        DavixError::clearError(&_last_error);
 }
 
 
 void configureSession(ne_session *_sess, const Uri & _u, const RequestParams &params, ne_auth_creds lp_callback, void* lp_userdata,
-                      ne_ssl_provide_fn cred_callback,  void* cred_userdata){
+                      ne_ssl_provide_fn cred_callback,  void* cred_userdata, bool & reused){
 
     void* state = ne_get_session_private(_sess,davix_neon_key);
+    if(state != NULL){
+        reused = true;
+    }
+
     if(state == NULL || state != params.getParmState()){
         // no configuration done, need to configure
-        DAVIX_TRACE("NEONSession : configure session...");
+        DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "configure session...");
 
         if(strcmp(ne_get_scheme(_sess), "https") ==0) // fix a libneon bug with non ssl connexion
             ne_ssl_trust_default_ca(_sess);
@@ -171,49 +181,29 @@ void configureSession(ne_session *_sess, const Uri & _u, const RequestParams &pa
         ne_set_useragent(_sess, params.getUserAgent().c_str());
 
         if(params.getSSLCACheck() == false){ // configure ssl check
-            DAVIX_DEBUG("NEONRequest : disable ssl verification");
+            DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "disable ssl verification");
             ne_ssl_set_verify(_sess, validate_all_certificate, NULL);
         }
 
-        // if authentification for login/password
-        if( params.getClientLoginPassword().first.empty() == false
-                || _u.getUserInfo().size() > 0
-                || params.getClientLoginPasswordCallback().first != NULL){
-            DAVIX_DEBUG("NEONSession : enable login/password authentication");
-            ne_set_server_auth(_sess, lp_callback, lp_userdata);
-        }else{
-            DAVIX_DEBUG("NEONSession : disable login/password authentication");
-        }
-
-        // if authentification for cli cert by callback
-        if( params.getClientCertCallbackX509().first != NULL){
-            DAVIX_DEBUG("NEONSession : enable client cert authentication by callback ");
-            ne_ssl_provide_clicert(_sess, cred_callback, cred_userdata);
-        }else if( params.getClientCertX509().hasCert()){
-            ne_ssl_set_clicert(_sess, X509CredentialExtra::extract_ne_ssl_clicert(params.getClientCertX509()));
-            DAVIX_DEBUG("NEONSession : enable client cert authentication with predefined cert");
-        }else{
-              DAVIX_DEBUG("NEONSession : disable client cert authentication");
-        }
-
-        if( timespec_isset(params.getOperationTimeout())){
-            const int timeout = static_cast<int>(params.getOperationTimeout()->tv_sec);
-            DAVIX_DEBUG("NEONSession : define operation timeout to %d", timeout);
-            ne_set_read_timeout(_sess, timeout);
-        }
         if(timespec_isset(params.getConnectionTimeout())){
             const int timeout = static_cast<int>(params.getConnectionTimeout()->tv_sec);
-            DAVIX_DEBUG("NEONSession : define connection timeout to %d" , timeout);
+            DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "define connection timeout to {}" , timeout);
             ne_set_connect_timeout(_sess, timeout);
         }
 
+        if(timespec_isset(params.getOperationTimeout())){
+            const int timeout = static_cast<int>(params.getOperationTimeout()->tv_sec);
+            DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "define operation timeout to {}" , timeout);
+            ne_set_read_timeout(_sess, timeout);
+        }
+
         for(std::vector<std::string>::const_iterator it = params.listCertificateAuthorityPath().begin(); it < params.listCertificateAuthorityPath().end(); it++){
             struct stat st;
             if ( stat(it->c_str(), &st) < 0 || S_ISDIR(st.st_mode) == false){
-                DAVIX_LOG(DAVIX_LOG_WARNING, "NEONSession : CA Path invalid : %s, %s ", it->c_str(), (errno != 0)?strerror(errno):strerror(ENOTDIR));
+                DAVIX_SLOG(DAVIX_LOG_WARNING, DAVIX_LOG_HTTP, "CA Path invalid : {}, {} ", *it, ((errno != 0)?strerror(errno):strerror(ENOTDIR)));
                 errno = 0;
             }else{
-                DAVIX_TRACE(" add CA PATH %s", it->c_str());
+                DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, "add CA PATH {}", *it);
                 ne_ssl_truse_add_ca_path(_sess, it->c_str());
             }
         }
@@ -223,6 +213,24 @@ void configureSession(ne_session *_sess, const Uri & _u, const RequestParams &pa
         // setup sess key
         ne_set_session_private(_sess, davix_neon_key, params.getParmState());
     }
+    // configure callback for new request
+    if( params.getClientLoginPassword().first.empty() == false
+            || _u.getUserInfo().size() > 0
+            || params.getClientLoginPasswordCallback().first != NULL){
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "enable login/password authentication");
+        ne_set_server_auth(_sess, lp_callback, lp_userdata);
+    }else{
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "disable login/password authentication");
+    }
+
+    // if authentification for cli cert by callback
+    if( params.getClientCertFunctionX509()){
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "enable client cert authentication by callback ");
+        ne_ssl_provide_clicert(_sess, cred_callback, cred_userdata);
+    }else{
+          DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "disable client cert authentication");
+    }
+
 
 }
 
