@@ -27,9 +27,6 @@
 
 namespace Davix {
 
-const char* proto_support[] = { "http", "https", NULL };
-const unsigned int ports[] = { 80 ,443 , 0};
-
 static boost::once_flag neon_once = BOOST_ONCE_INIT;
 
 static void init_neon(){
@@ -53,8 +50,8 @@ NEONSessionFactory::NEONSessionFactory() :
     _redirCache(256)
 {
     boost::call_once(&init_neon, neon_once);
-    DAVIX_TRACE("HTTP/SSL Session caching %s", (_session_caching?"ENABLED":"DISABLED"));
-    DAVIX_TRACE("Redirection Session caching %s", (_redir_caching?"ENABLED":"DISABLED"));
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CORE, "HTTP/SSL Session caching {}", (_session_caching?"ENABLED":"DISABLED"));
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CORE, "Redirection Session caching {}", (_redir_caching?"ENABLED":"DISABLED"));
 }
 
 NEONSessionFactory::~NEONSessionFactory(){
@@ -88,11 +85,11 @@ int NEONSessionFactory::createNeonSession(const RequestParams & params, const Ur
         }
     }
 
-    DavixError::setupError(err, davix_scope_http_request(), StatusCode::UriParsingError, "impossible to parse " + uri.getString() + ", not a valid HTTP, S3 or Webdav URL");
+    DavixError::setupError(err, davix_scope_http_request(), StatusCode::UriParsingError, fmt::format("impossible to parse {}, not a valid HTTP, S3 or Webdav URL", uri.getString()));
     return -1;
 }
 
-int NEONSessionFactory::storeNeonSession(ne_session* sess, DavixError **err){
+int NEONSessionFactory::storeNeonSession(ne_session* sess){
     internal_release_session_handle(sess);
     return 0;
 }
@@ -106,7 +103,7 @@ ne_session* NEONSessionFactory::create_session(const RequestParams & params, con
 
     const Uri* proxy = params.getProxyServer();
     if(se != NULL && proxy != NULL){
-        DAVIX_TRACE(" configure mandatory proxy to %s", proxy->getString().c_str());
+        DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_HTTP, " configure mandatory proxy to {}", proxy->getString().c_str());
         const enum ne_sock_sversion version = ((proxy->getProtocol().compare("socks5") ==0)?NE_SOCK_SOCKSV5:NE_SOCK_SOCKSV4);
         const int port_proxy = ((proxy->getPort() ==0)?1080:(proxy->getPort()));
         const std::string & userinfo = proxy->getUserInfo();
@@ -128,19 +125,19 @@ ne_session* NEONSessionFactory::create_session(const RequestParams & params, con
 
 ne_session* NEONSessionFactory::create_recycled_session(const RequestParams & params, const std::string &protocol, const std::string &host, unsigned int port){
 
-    ne_session* se= NULL;
-    {
+    if(params.getKeepAlive()){
+        ne_session* se= NULL;
         boost::lock_guard<boost::mutex> lock(_sess_mut);
         std::multimap<std::string, ne_session*>::iterator it;
         if( (it = _sess_map.find(create_map_keys_from_URL(protocol, host, port))) != _sess_map.end()){
-            DAVIX_DEBUG("cached ne_session found ! taken from cache ");
+            DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "cached ne_session found ! taken from cache ");
             se = it->second;
             _sess_map.erase(it);
             return se;
         }
 
     }
-    DAVIX_DEBUG("no cached ne_session, create a new one ");
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "no cached ne_session, create a new one ");
     return create_session(params, protocol, host, port);
 }
 
@@ -157,13 +154,13 @@ void NEONSessionFactory::internal_release_session_handle(ne_session* sess){
     std::string sess_key;
     sess_key.append(ne_get_scheme(sess)).append(ne_get_server_hostport(sess));
 
-    DAVIX_DEBUG("add old session to cache %s", sess_key.c_str());
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "add old session to cache {}", sess_key.c_str());
 
     _sess_map.insert(std::pair<std::string, ne_session*>(sess_key, sess));
 }
 
-static std::string redirectionCreateKey(const std::string & method, const Uri & origin){
-    static const std::string head_method("HEAD");
+static const std::string redirectionCreateKey(const std::string & method, const Uri & origin){
+    const std::string head_method("HEAD");
     std::string res;
 
     res.reserve(method.size() + origin.getString().size() +3);
@@ -174,16 +171,14 @@ static std::string redirectionCreateKey(const std::string & method, const Uri &
 
 void NEONSessionFactory::addRedirection( const std::string & method, const Uri & origin, boost::shared_ptr<Uri> dest){
     if(_redir_caching){
-        DAVIX_DEBUG("Cache redirection <%s %s %s>", method.c_str(), origin.getString().c_str(), dest->getString().c_str());
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "Add cached redirection <{} {} {}>", method.c_str(), origin.getString().c_str(), dest->getString().c_str());
         _redirCache.insert(redirectionCreateKey(method, origin), dest);
     }
 }
 
 boost::shared_ptr<Uri> NEONSessionFactory::redirectionResolve(const std::string & method, const Uri & origin){
-    boost::shared_ptr<Uri> res =  _redirCache.find(redirectionCreateKey(method, origin));
-
+    boost::shared_ptr<Uri> res = redirectionResolveSingle(method, origin);
     if(res.get() != NULL){
-        DAVIX_DEBUG("Cached redirection found <%s %s %s>", method.c_str(), origin.getString().c_str(), res->getString().c_str());
         boost::shared_ptr<Uri> res_rec = redirectionResolve(method, *res);
         if(res_rec.get() != NULL)
             return res_rec;
@@ -191,26 +186,40 @@ boost::shared_ptr<Uri> NEONSessionFactory::redirectionResolve(const std::string
     return res;
 }
 
-void NEONSessionFactory::redirectionClean(const std::string & method, const Uri & origin){
-    boost::shared_ptr<Uri> res = redirectionResolve(method, origin);
-    if(res.get() != NULL){
-        redirectionClean(method, *res);
-    }
-    DAVIX_DEBUG("Delete Cached redirections for <%s %s>", method.c_str(), origin.getString().c_str());
-    _redirCache.erase(redirectionCreateKey(method, origin));
+boost::shared_ptr<Uri> NEONSessionFactory::redirectionResolveSingleIntern(const std::string & method, const Uri & origin){
+    return  _redirCache.find(redirectionCreateKey(method, origin));
 }
 
 
-std::string create_map_keys_from_URL(const std::string & protocol, const std::string &host, unsigned int port){
-    std::ostringstream oss;
-    if( (strcmp(protocol.c_str(), "http") ==0 && port == 80)
-            || ( strcmp(protocol.c_str(), "https") ==0 && port == 443)){
-      oss <<  protocol << host;
-    }else
-        oss <<  protocol << host << ":" << port;
-    std::string res = oss.str();
-    DAVIX_DEBUG(" creating session keys... %s", res.c_str());
+boost::shared_ptr<Uri> NEONSessionFactory::redirectionResolveSingle(const std::string & method, const Uri & origin){
+    boost::shared_ptr<Uri> res = redirectionResolveSingleIntern(method, origin);
+    if(res.get() != NULL){
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "Found redirection  <{} {} {}>", method.c_str(), origin.getString().c_str(), res->getString().c_str());
+    }
     return res;
 }
 
+void NEONSessionFactory::redirectionClean(const std::string & method, const Uri & origin){
+    boost::shared_ptr<Uri> res = redirectionResolveSingleIntern(method, origin);
+    if(res.get() != NULL){
+        DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "Delete Cached redirection for <{} {} {}>", method.c_str(), origin.getString().c_str(), res->getString().c_str());
+        _redirCache.erase(redirectionCreateKey(method, origin));
+        redirectionClean(method, *res);
+    }
+}
+
+
+
+
+std::string create_map_keys_from_URL(const std::string & protocol, const std::string &host, unsigned int port){
+    std::string host_port;
+    if( (strcmp(protocol.c_str(), "http") ==0 && port == 80)
+            || ( strcmp(protocol.c_str(), "https") ==0 && port == 443)){
+        host_port = fmt::format("{}{}", protocol, host);
+    }else
+        host_port = fmt::format("{}{}:{}", protocol, host, port);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, " creating session keys... {}", host_port);
+    return host_port;
+}
+
 } // namespace Davix
