@@ -39,9 +39,9 @@ HttpRequest::HttpRequest(NEONRequest* req) : d_ptr(req)
 HttpRequest::HttpRequest(Context & context, const Uri & uri, DavixError** err) :
     d_ptr(new NEONRequest(*this, context,uri)){
 
-    DAVIX_DEBUG("Creat HttpRequest for %s", uri.getString().c_str());
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_HTTP, "Creat HttpRequest for {}", uri.getString());
     if(uri.getStatus() != StatusCode::OK){
-        DavixError::setupError(err, davix_scope_http_request(), StatusCode::UriParsingError, "impossible to parse " + uri.getString() + " ,not a valid HTTP or Webdav URL");
+        DavixError::setupError(err, davix_scope_http_request(), StatusCode::UriParsingError, fmt::format(" {} is not not a valid HTTP or Webdav URL", uri));
     }
 }
 
@@ -50,13 +50,13 @@ HttpRequest::HttpRequest(Context & context, const std::string & url, DavixError*
     Uri uri(url);
     d_ptr= new NEONRequest(*this, context, uri);
     if(uri.getStatus() != StatusCode::OK){
-        DavixError::setupError(err, davix_scope_http_request(), StatusCode::UriParsingError, "impossible to parse " + uri.getString() + " ,not a valid HTTP or Webdav URL");
+        DavixError::setupError(err, davix_scope_http_request(), StatusCode::UriParsingError, fmt::format(" {} is not not a valid HTTP or Webdav URL", uri));
     }
 }
 
 HttpRequest::~HttpRequest()
 {
-    DAVIX_DEBUG("Destroy HttpRequest");
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CORE, "Destroy HttpRequest");
     delete d_ptr;
 }
 
@@ -133,12 +133,12 @@ dav_ssize_t HttpRequest::readSegment(char* buffer, dav_size_t max_size, DavixErr
 }
 
 dav_ssize_t HttpRequest::readLine(char *buffer, dav_size_t max_size, DavixError **err){
-    DAVIX_DEBUG("Davix::Request::readLine want to read a line of max %lld chars", max_size);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CORE, "Davix::Request::readLine want to read a line of max {} chars", max_size);
     dav_ssize_t ret= -1;
     TRY_DAVIX{
         if( (ret =  d_ptr->readLine(buffer, max_size, err)) >= 0){
-            DAVIX_DEBUG("Davix::Request::readLine got %lld chars", ret);
-            DAVIX_TRACE("Davix::Request::readLine content\n{{%.*s}}\n", ret, buffer);
+            DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CORE, "Davix::Request::readLine got {} chars", ret);
+            DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CORE, "Davix::Request::readLine content\n[[{}]]\n", std::string(buffer, ret));
         }
     }CATCH_DAVIX(err)
     return ret;
@@ -193,6 +193,11 @@ const char* HttpRequest::getAnswerContent(){
     return d_ptr->getAnswerContent();
 }
 
+std::vector<char> & HttpRequest::getAnswerContentVec(){
+    return d_ptr->getAnswerContentVec();
+}
+
+
 /// get content length
 dav_ssize_t HttpRequest::getAnswerSize() const{
     return d_ptr->getAnswerSize();
@@ -203,6 +208,7 @@ HttpCacheToken* HttpRequest::extractCacheToken()const{
 }
 
 void HttpRequest::useCacheToken(const HttpCacheToken *token){
+    (void) token;
 }
 
 /// set a HttpRequest flag
@@ -286,10 +292,8 @@ PropfindRequest::PropfindRequest(Context & context, const Uri & uri, DavixError*
 }
 
 
-
-void httpcodeToDavixCode(int code, const std::string & scope, const std::string & end_message, DavixError** err){
-    StatusCode::Code dav_code;
-    std::string str_msg;
+void httpcodeToDavixError(int code, const std::string &scope, const std::string & end_message, StatusCode::Code & dav_code, std::string & err_msg){
+    char const* str_msg = "Status Ok";
     switch (code) {
         case 200:           /* OK */
         case 201:           /* Created */
@@ -301,44 +305,48 @@ void httpcodeToDavixCode(int code, const std::string & scope, const std::string
         case 207:           /* Multi-Status */
         case 304:           /* Not Modified */
             dav_code = StatusCode::OK;
-            str_msg ="Status Ok";
+            str_msg = "Success";
             break;
         case 401:           /* Unauthorized */
         case 402:           /* Payment Required */
         case 407:           /* Proxy Authentication Required */
             dav_code = StatusCode::AuthentificationError;
-            str_msg = "HTTP Authentification Error";
+            str_msg = "Authentification Error";
             break;
         case 303:           /* See Other */
         case 404:           /* Not Found */
         case 410:           /* Gone */
             dav_code = StatusCode::FileNotFound;
-            str_msg = "HTTP File not found";
+            str_msg = "File not found";
             break;
         case 408:           /* Request Timeout */
         case 504:           /* Gateway Timeout */
             dav_code = StatusCode::OperationTimeout;
-            str_msg = "HTTP Operation timeout";
+            str_msg = "Operation timeout";
             break;
         case 409:
-            if (scope == davix_scope_mkdir_str())
+            if (scope == davix_scope_mkdir_str()){
                 dav_code = StatusCode::FileNotFound;
-            else
+                str_msg = "Conflict, File not Found";
+            } else{
                 dav_code = StatusCode::FileExist;
-            str_msg = "HTTP Conflict";
+                str_msg = "Conflict, File Exist";
+            }
             break;
         case 423:           /* Locked */
         case 403:           /* Forbidden */
             dav_code = StatusCode::PermissionRefused;
-            str_msg = "HTTP Permission refused";
+            str_msg = "Permission refused";
             break;
         break;
         case 405:           /* Method Not Allowed */
-            if (scope == davix_scope_mkdir_str())
+            if (scope == davix_scope_mkdir_str()){
                 dav_code = StatusCode::FileExist;
-            else
+                str_msg = "Method Not Allowed, File Exist";
+            } else{
                 dav_code = StatusCode::PermissionRefused; // Technically is a EPERM
-            str_msg = "Method Not Allowed";
+                str_msg = "Method Not Allowed, Permission refused";
+            }
             break;
         case 400:           /* Bad Request */
         case 411:           /* Length Required */
@@ -350,7 +358,7 @@ void httpcodeToDavixCode(int code, const std::string & scope, const std::string
         case 413:           /* Request Entity Too Large */
         case 507:           /* Insufficient Storage */
             dav_code = StatusCode::ConnectionProblem;
-            str_msg = "HTTP Server Error";
+            str_msg = "Server Error";
             break;
         case 301:           /* Moved Permanently */
         case 300:           /* Multiple Choices */
@@ -371,13 +379,26 @@ void httpcodeToDavixCode(int code, const std::string & scope, const std::string
         case 505:           /* HTTP Version Not Supported */
         default:
             dav_code = StatusCode::UnknowError;
-            str_msg = "HTTP unexcepted Server Error";
+            str_msg = "Unexcepted Server Error";
             break;
     }
 
-    std::ostringstream ss;
-    ss << "HTTP Error "<< code << " " << str_msg << std::endl;
-    DavixError::setupError(err, scope, dav_code, ss.str());
+    err_msg.assign(fmt::format("HTTP {} : {} {}", code, str_msg, end_message));
+}
+
+void httpcodeToDavixError(int code, const std::string & scope, const std::string & end_message, DavixError** err){
+    StatusCode::Code davix_code;
+    std::string err_msg;
+    httpcodeToDavixError(code, scope, end_message, davix_code, err_msg);
+    DavixError::setupError(err, scope, davix_code, err_msg);
+}
+
+
+void httpcodeToDavixException(int code, const std::string & scope, const std::string & end_message){
+    StatusCode::Code davix_code;
+    std::string err_msg;
+    httpcodeToDavixError(code, scope, end_message, davix_code, err_msg);
+    throw DavixException(scope, davix_code, err_msg);
 }
 
 bool httpcodeIsValid(int code)
