@@ -104,7 +104,7 @@ void DavixError::setErrScope(const std::string &scope){
 void DavixError::setupError(DavixError **err, const std::string &scope, StatusCode::Code errCode, const std::string &errMsg){
     if(err){
         if(*err){
-            DAVIX_DEBUG("Error Override of previous DavixError, BUG !");
+            DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CORE, "Error Override of previous DavixError, BUG !");
             // error msg
         }
         *err = new DavixError(scope, errCode, errMsg);
@@ -128,6 +128,7 @@ void DavixError::propagatePrefixedError(DavixError **newErr, DavixError *oldErr,
             *newErr = oldErr;
             if(*newErr && prefix.empty() == false){
                 std::string new_mess(prefix);
+                new_mess.append(" ");
                 (*newErr)->d_ptr->_errMsg = new_mess.append((*newErr)->d_ptr->_errMsg);
             }
         }
@@ -146,6 +147,13 @@ void DavixError::clearError(DavixError **err){
     }
 }
 
+
+void errno_to_davix_exception(int errno_code, const std::string &scope, const std::string &msg){
+    DavixError* tmp_err=NULL;
+    davix_errno_to_davix_error(errno_code, scope, msg, &tmp_err);
+    checkDavixError(&tmp_err);
+}
+
 void davix_errno_to_davix_error(int errcode, const std::string & scope, const std::string & msg, DavixError** newErr){
     StatusCode::Code c;
     std::string msg_final;
@@ -243,11 +251,19 @@ void DavixException::toDavixError(DavixError **err){
 
 
 std::string davix_scope_stat_str(){
-    return "Davix::Posix::stat";
+    return "Davix::stat";
 }
 
 std::string davix_scope_mkdir_str(){
-    return "Davix::Posix::mkdir";
+    return "Davix::mkdir";
+}
+
+std::string davix_scope_rm_str(){
+    return "Davix::rm";
+}
+
+std::string davix_scope_mv_str(){
+    return "Davix::mv";
 }
 
 std::string davix_scope_directory_listing_str(){
@@ -282,6 +298,8 @@ std::string davix_scope_x509cred(){
     return "Davix::X509cred";
 }
 
+
+
 } // namespace Davix
 
 
