@@ -13,6 +13,7 @@ const std::string scope_main = "Davix::Tools::davix";
 
 static void performanceCallback(const PerformanceData& perfData, void *udata)
 {
+    (void) udata;
     std::cout << perfData.totalTransferred()
               << " (" << perfData.avgTransfer() << " bytes/sec)"
               << std::endl;
@@ -20,7 +21,7 @@ static void performanceCallback(const PerformanceData& perfData, void *udata)
 
 
 static std::string help_msg(){
-    return Tool::get_base_description_options() +
+    return Tool::get_copy_description_options() +
             Tool::get_common_options();
 }
 
@@ -34,7 +35,8 @@ int main(int argc, char** argv){
 
     if( (retcode= Tool::parse_davix_options(argc, argv, opts, &tmp_err)) ==0){
         Context c;
-        if( (retcode = Tool::configureAuth(opts, &tmp_err)) == 0){
+        if( (retcode = Tool::configureAuth(opts)) == 0){
+            configureContext(c, opts);
             DavixCopy copy(c, &opts.params);
             copy.setPerformanceCallback(performanceCallback, NULL);
             copy.copy(opts.vec_arg[0], opts.vec_arg[1],
