@@ -18,6 +18,7 @@
  *
 */
 
+
 #include <davix_internal.hpp>
 #include "davix_tool_params.hpp"
 #include "davix_tool_util.hpp"
@@ -41,7 +42,12 @@ const std::string scope_params = "Davix::Tools::Params";
 #define S3_SECRET_KEY       1005
 #define S3_ACCESS_KEY       1006
 #define X509_PRIVATE_KEY    1007
-#define HEADERS_OPTIONS     1008
+#define TRACE_OPTIONS       1008
+#define REDIRECTION_OPT     1009
+#define METALINK_OPT        1010
+#define CONN_TIMEOUT        1011
+#define TIMEOUT_OPS         1012
+#define RETRY_OPT           1013
 
 // LONG OPTS
 
@@ -49,9 +55,14 @@ const std::string scope_params = "Davix::Tools::Params";
 {"debug", no_argument, 0,  DEBUG_OPT }, \
 {"header",  required_argument, 0,  'H' }, \
 {"help", no_argument, 0,'?'}, \
+{"metalink", required_argument, 0, METALINK_OPT }, \
 {"module", required_argument, 0, 'P'}, \
 {"proxy", required_argument, 0, 'x'}, \
-{"trace-headers", no_argument, 0, HEADERS_OPTIONS }, \
+{"redirection", required_argument, 0, REDIRECTION_OPT }, \
+{"conn-timeout", required_argument, 0, CONN_TIMEOUT }, \
+{"retry", required_argument, 0, RETRY_OPT }, \
+{"timeout", required_argument, 0, TIMEOUT_OPS }, \
+{"trace", required_argument, 0, TRACE_OPTIONS }, \
 {"verbose", no_argument, 0,  0 }, \
 {"version", no_argument, 0, 'V'}
 
@@ -95,16 +106,26 @@ OptParams::OptParams() :
 
 
 static void option_abort(char** argv){
-    std::cerr << argv[0] <<", Error: Wrong argument format" << std::endl;
-    std::cerr << "try 'davix --help' for more informations" <<std::endl;
+    std::cerr << argv[0] <<", Error: Wrong argument format\n";
+    std::cerr << "Try '" << argv[0] <<" --help' for more informations" << std::endl;
     exit(-1);
 }
 
 static void display_version(){
-    std::cout << "Davix Version: " << version() << std::endl;
+    std::cout << "Version: " << version() << std::endl;
     exit(0);
 }
 
+template <typename T, typename Y, typename Z>
+Y match_option(T begin, T end,
+               Y begin_res, Y end_res,
+               Z val, char** argv){
+    Y res = match_array(begin, end, begin_res, end_res, val);
+    if(res == end_res){
+        option_abort(argv);
+    }
+    return res;
+}
 
 static int set_header_field(const std::string & arg, OptParams & p, DavixError** err){
     dav_size_t pos;
@@ -116,8 +137,43 @@ static int set_header_field(const std::string & arg, OptParams & p, DavixError**
     return 0;
 }
 
+static void set_metalink_opt(RequestParams & params, const std::string & metalink_opt, char** argv){
+    const std::string str_opt[] = { "no" , "disable", "auto", "failover" };
+    const Davix::MetalinkMode::MetalinkMode mode_opt[] = { MetalinkMode::Disable, MetalinkMode::Disable, MetalinkMode::Auto, MetalinkMode::FailOver };
+    params.setMetalinkMode(*match_option(str_opt, str_opt+sizeof(str_opt)/sizeof(str_opt[0]),
+                                               mode_opt, mode_opt + sizeof(mode_opt)/sizeof(mode_opt[0]),
+                                               metalink_opt, argv));
+}
 
 
+static void set_redirection_opt(RequestParams & params, const std::string & redir_opt, char** argv){
+    const std::string str_opt[] = { "no" , "yes", "auto"};
+    const bool mode_opt[] = { false, true, true };
+    params.setTransparentRedirectionSupport(*match_option(str_opt, str_opt+sizeof(str_opt)/sizeof(str_opt[0]),
+                                               mode_opt, mode_opt + sizeof(mode_opt)/sizeof(mode_opt[0]),
+                                               redir_opt, argv));
+}
+
+
+static int parse_int(const std::string & opt, char** argv){
+    try{
+        return toType<int, std::string>()(opt);
+    }catch(...){
+        std::cerr << "Invalid option value " << opt << std::endl;
+        option_abort(argv);
+    }
+    return 0;
+}
+
+static struct timespec parse_timeout(const std::string & opt, char** argv){
+    int t = parse_int(opt, argv);
+
+    struct timespec timelapse;
+    timelapse.tv_sec =t;
+    timelapse.tv_nsec =0;
+    return timelapse;
+}
+
 int parse_davix_options_generic(const std::string &opt_filter,
                                 const struct option* long_options,
                                 int argc, char** argv, OptParams & p, DavixError** err){
@@ -130,12 +186,13 @@ int parse_davix_options_generic(const std::string &opt_filter,
         switch(ret){
             case DEBUG_OPT:
                 p.debug = true;
-                davix_set_log_level(DAVIX_LOG_ALL);
+                // max log level
+                setLogLevel(DAVIX_LOG_TRACE);
+                setLogScope(DAVIX_LOG_SCOPE_ALL | DAVIX_LOG_SCOPE_NEON | DAVIX_LOG_BODY | DAVIX_LOG_XML);
                 break;
             case 'E':
-                 p.cred_path = std::string(optarg);
+                 p.cred_path = SanitiseTildedPath(optarg);
                  break;
-
             case 'k':
                 p.params.setSSLCAcheck(false);
                 break;
@@ -150,7 +207,7 @@ int parse_davix_options_generic(const std::string &opt_filter,
                 p.userlogpasswd.first = optarg;
                 break;
             case X509_PRIVATE_KEY:
-                p.priv_key = optarg;
+                p.priv_key = SanitiseTildedPath(optarg).c_str();
                 break;
             case USER_PASSWORD:
                 p.userlogpasswd.second = optarg;
@@ -158,6 +215,9 @@ int parse_davix_options_generic(const std::string &opt_filter,
             case DATA_CONTENT:
                 p.req_content = optarg;
                 break;
+            case RETRY_OPT:
+                p.params.setOperationRetry( parse_int(optarg, argv));
+                break;
             case S3_ACCESS_KEY:
                 p.aws_auth.second = optarg;
                 break;
@@ -176,8 +236,12 @@ int parse_davix_options_generic(const std::string &opt_filter,
             case 'V':
                 display_version();
                 return 1;
-            case HEADERS_OPTIONS:
-                p.pres_flag |= DISPLAY_HEADERS;
+            case TRACE_OPTIONS:
+                {
+                    setLogScope(0);
+                    setLogScope(std::string(optarg));
+                    setLogLevel(DAVIX_LOG_TRACE);
+                }
                 break;
             case 'x':
                 p.params.setProxyServer(std::string(optarg, 0, 2048));
@@ -185,6 +249,22 @@ int parse_davix_options_generic(const std::string &opt_filter,
             case 'X':
                 p.req_type = std::string(optarg, 0, 255);
                 break;
+            case METALINK_OPT:
+                 set_metalink_opt(p.params, std::string(optarg), argv);
+                 break;
+            case REDIRECTION_OPT:
+                 set_redirection_opt(p.params, std::string(optarg), argv);
+                 break;
+            case CONN_TIMEOUT:{
+                struct timespec s =parse_timeout(std::string(optarg), argv);
+                p.params.setConnectionTimeout(&s);
+                }
+                break;
+            case TIMEOUT_OPS:{
+                struct timespec s =parse_timeout(std::string(optarg), argv);
+                p.params.setOperationTimeout(&s);
+                break;
+             }
             case '?':
             printf(p.help_msg.c_str(), argv[0]);
                 return -1;
@@ -295,23 +375,27 @@ int parse_davix_put_options(int argc, char** argv, OptParams & p, DavixError** e
 const std::string  & get_common_options(){
     static const std::string s(
             "  Common Options:\n"
+            "\t--conn-timeout TIME:      Connection timeout in seconds. default: 30\n"
             "\t--debug:                  Debug mode\n"
             "\t--header, -H:             Add a header field to the request\n"
             "\t--help, -h:               Display this help message\n"
-            "\t--module, -P name:        Load a plugin or profile by name\n"
-            "\t--proxy, -x url:          SOCKS5 proxy server URL (ex: socks5://login:pass@socks.example.org )\n"
-            "\t--trace-headers:          Trace all HTTP queries headers\n"
+            "\t--metalink OPT:           Metalink support. value=failover|no. default=failover) \n"
+            "\t--module, -P NAME:        Load a plugin or profile by name\n"
+            "\t--proxy, -x URL:          SOCKS5 proxy server URL. (Ex: socks5://login:pass@socks.example.org)\n"
+            "\t--redirection OPT:        Transparent redirection support. value=yes|no. default=yes)\n"
+            "\t--timeout TIME:           Global timeout for the operation in seconds. default: infinite\n"
+            "\t--trace:                  Specify one or more scpoes to trace. (Ex: --trace log level(optional),header,file)\n"
             "\t--verbose:                Verbose mode\n"
             "\t--version, -V:            Display version\n"
             "  Security Options:\n"
-            "\t--capath CA_path:         Add an additional certificate authority directory\n"
-            "\t--cert, -E cred_path:     Client Certificate in PEM format\n"
-            "\t--key priv_path:          Private key in PEM format\n"
+            "\t--capath CA_PATH:         Add an additional certificate authority directory\n"
+            "\t--cert, -E CRED_PATH:     Client Certificate in PEM format\n"
+            "\t--key KEY_PATH:           Private key in PEM format\n"
             "\t--insecure, -k:           Disable SSL credential checks\n"
             "\t--userlogin:              User login for login/password authentication\n"
             "\t--userpass:               User password for login/password authentication\n"
-            "\t--s3secretkey:            S3 authentication: secret key\n"
-            "\t--s3accesskey:            S3 authentication: access key\n"
+            "\t--s3secretkey SEC_KEY:    S3 authentication: secret key\n"
+            "\t--s3accesskey ACC_KEY:    S3 authentication: access key\n"
             );
     return s;
 }
@@ -319,19 +403,24 @@ const std::string  & get_common_options(){
 
 
 const std::string  & get_base_description_options(){
-    static const std::string s("Usage: %s [OPTIONS ...] <url>\n"
+    static const std::string s("Usage: {} [OPTIONS ...] <url>\n"
             );
     return s;
 }
 
 
 const std::string  & get_put_description_options(){
-    static const std::string s("Usage: %s [OPTIONS ...] <local_file> <remote_file_url> \n"
+    static const std::string s("Usage: {} [OPTIONS ...] <local_file> <remote_file_url> \n"
             );
     return s;
 }
 
 
+const std::string  & get_copy_description_options(){
+    static const std::string s("Usage: {} [OPTIONS ...] <src_url> <dst_url>\n"
+            );
+    return s;
+}
 
 }
 
