@@ -23,6 +23,7 @@
 #include <tools/davix_tool_util.hpp>
 #include <cstdio>
 
+
 // @author : Devresse Adrien
 // main file for davix-put operation
 
@@ -48,17 +49,20 @@ static int execute_put(const Tool::OptParams & opts, int fd, DavixError** err){
         const std::string &  dst_file = opts.vec_arg[1];
         Context c;
         configureContext(c, opts);
-        DavFile f(c, dst_file);
-        struct stat st;
-        if( fstat(fd, &st) != 0){
-            davix_errno_to_davix_error(errno, scope_put, std::string("for source file ").append(src_file), err);
-            return -1;
-        }
-        if( S_ISREG(st.st_mode) && st.st_size > 0){
-            return f.putFromFd(&opts.params, fd, static_cast<dav_size_t>(st.st_size), err);
-        }
-        DavixError::setupError(err, scope_put, StatusCode::SystemError, std::string(dst_file).append("is not a valid regular file"));
-        return -1;
+
+        TRY_DAVIX{
+            DavFile f(c, dst_file);
+            struct stat st;
+            if( fstat(fd, &st) != 0){
+                errno_to_davix_exception(errno, scope_put, std::string("for source file ").append(src_file));
+            }
+            if( S_ISREG(st.st_mode) && st.st_size > 0){
+                    f.put(&opts.params, fd, static_cast<dav_size_t>(st.st_size));
+                    return 0;
+            }
+            throw DavixException(scope_put, StatusCode::SystemError, std::string(dst_file).append("is not a valid regular file"));
+      }CATCH_DAVIX(err);
+      return -1;
 }
 
 
@@ -72,12 +76,15 @@ int main(int argc, char** argv){
     int fd_in= -1;
 
     if( (retcode= Tool::parse_davix_put_options(argc, argv, opts, &tmp_err)) ==0
-        && (retcode = Tool::configureAuth(opts, &tmp_err)) == 0){
+        && (retcode = Tool::configureAuth(opts)) == 0
+        && (retcode = Tool::configureMonitorCB(opts, Transfer::Write)) == 0){
         if( ( fd_in = Tool::getInFd(opts, scope_put, &tmp_err)) > 0){
             retcode = execute_put(opts, fd_in, &tmp_err);
             close(fd_in);
         }
     }
+    
+
     Tool::errorPrint(&tmp_err);
     return retcode;
 }
