@@ -18,21 +18,25 @@
  *
 */
 
+
 #include <utils/davix_types.hpp>
 #include <davix.hpp>
 #include <davix_internal.hpp>
 #include <string_utils/stringutils.hpp>
 #include "davix_tool_util.hpp"
 
-
+#include <ctype.h>
 #include <simple_getpass/simple_get_pass.h>
 
+#include <sys/ioctl.h>
+#include <unistd.h>
 
 namespace Davix{
 
 namespace Tool{
 
 using namespace StrUtil;
+using namespace std::placeholders;
 
 dav_ssize_t writeToFd(int fd, const void* buffer, dav_size_t s_buff){
     while(1){
@@ -53,7 +57,7 @@ std::string string_from_ptime(const time_t & t){
     char b[255];
     b[sizeof(b)-1]= '\0';
 
-    struct tm local_time={0};
+    struct tm local_time;
     (void) localtime_r(&t, &local_time);
     strftime(b, 254, "%F %T", &local_time);
     return std::string(b);
@@ -89,7 +93,7 @@ void writeConsoleLine(int fd, char symbol,  const std::string & msg){
     writeToFd(fd, ss.str());
 }
 
-int configureAuth(OptParams & opts, DavixError** err){
+int configureAuth(OptParams & opts){
     // setup client side credential
     opts.params.setClientCertCallbackX509(&authCallbackCert, &opts);
     // setup client login / password
@@ -108,7 +112,7 @@ int configureAuth(OptParams & opts, DavixError** err){
 static void printHookHeaders(char symbol, const std::string & first_msg, const std::string & start_line){
     std::string req_header(start_line);
     StrUtil::remove(req_header, '\r');
-    rtrim(req_header, StrUtil::isCrLf);
+    rtrim(req_header, StrUtil::isCrLf());
     std::vector<std::string> res;
     split(req_header, '\n', res);
 
@@ -122,7 +126,7 @@ static void printHookHeadersVec(char symbol, const std::string & first_msg, cons
                                 const HeaderVec & headers){
     std::string parsed_first_line(init_line);
     StrUtil::remove(parsed_first_line, '\r');
-    rtrim(parsed_first_line, StrUtil::isCrLf);
+    rtrim(parsed_first_line, StrUtil::isCrLf());
 
     writeConsoleLine(STDOUT_FILENO, '*', first_msg);
     writeConsoleLine(STDOUT_FILENO, symbol, parsed_first_line);
@@ -134,10 +138,13 @@ static void printHookHeadersVec(char symbol, const std::string & first_msg, cons
 }
 
 void hook_davix_tool_pre_send(HttpRequest& req, const std::string & start_line){
+    (void) req;
     printHookHeaders('>', "Send request", start_line);
 }
 
 void hook_davix_tool_pre_rec(HttpRequest& req, const std::string & init_line, const HeaderVec & headers, int status_code){
+    (void) req;
+    (void) status_code;
     printHookHeadersVec('<', "Receive answer", init_line, headers);
 }
 
@@ -207,6 +214,7 @@ std::string mode_to_stringmode(mode_t mode){
 
 int authCallbackLoginPassword(void* userdata, const SessionInfo & info, std::string & login, std::string & password,
                                         int count, DavixError** err){
+    (void) info;
     OptParams* opts = static_cast<OptParams*>(userdata);
     int ret = -1;
     if(opts->userlogpasswd.first.empty() == false){
@@ -233,25 +241,33 @@ int authCallbackLoginPassword(void* userdata, const SessionInfo & info, std::str
 }
 
 int authCallbackCert(void* userdata, const SessionInfo & info, X509Credential* cert, DavixError** err){
+    (void) info;
     OptParams* opts = static_cast<OptParams*>(userdata);
-    int ret = -1;
     const std::string key_path(opts->priv_key), cred_path(opts->cred_path);
 
-
     if(cred_path.empty() == false){
-        if( cert->loadFromFilePEM( ((key_path.empty()== false)?(key_path):(cred_path)),
+        // try without password
+        if(  cert->loadFromFilePEM( ((key_path.empty()== false)?(key_path):(cred_path)),
                                   cred_path,
                                   "",
                                   err) <0){
+            
+            if( (*err)->getStatus() != StatusCode::CredDecryptionError){
+                // credential specific error
+                return -1;
+            }
 
-            // FIX IT : Neon SSL API does not allow to know if error is bad path  or wrong password, try again with password
+            // retry with password
+            std::cout << std::endl;
+            DavixError::clearError(err);
             std::string password;
             if( ask_user_passwd(password) <0
                     || cert->loadFromFilePEM(key_path, cred_path, password, err) <0 ){
                 if(err && *err == NULL){
                     DavixError::setupError(err, "Davix::Tool::Auth",
-                                           StatusCode::LoginPasswordError,
-                                           "Impossible to use client credential");
+                                           StatusCode::CredDecryptionError,
+                                           "Impossible to use and decrypt client credential");
+
                 }
                 return -1;
              }
@@ -259,10 +275,11 @@ int authCallbackCert(void* userdata, const SessionInfo & info, X509Credential* c
         std::cout << std::endl;
         return 0;
     }
-    if(ret < 0)
+    if(err && *err == NULL){
         DavixError::setupError(err, "Davix::Tool::Auth",
                                StatusCode::LoginPasswordError,
                                "No valid client credential provided");
+    }
     return -1;
 
 }
@@ -288,14 +305,6 @@ std::string string_from_size_t(size_t number, size_t size_string){
     return ss.str();
 }
 
-
-std::string filename_from_uri(const std::string & current_dir, const Uri & uri){
-    if(uri.getStatus() == StatusCode::OK){
-
-    }
-    return std::string();
-}
-
 bool isShell(int fd){
     if(isatty(fd) ==1)
         return true;
@@ -309,5 +318,143 @@ void flushFinalLineShell(int fd){
     }
 }
 
+std::string SanitiseTildedPath(const char * path){
+    if(path[0] == '~')
+    {
+        std::string newpath(path);
+        newpath.erase(0, 1);
+        const char * val = std::getenv("HOME");
+        
+        if(val != NULL){
+            newpath.insert(0, val);
+        }
+        else{
+            newpath.insert(0, "/");
+        }
+        return newpath;
+    }   
+    return std::string(path);
+}
+
+bool is_number(const std::string& s)
+{
+    std::string::const_iterator it = s.begin();
+    while (it != s.end() && std::isdigit(*it)) ++it;
+    return !s.empty() && it == s.end();
+}
+
+
+void TransferMonitor(const Uri & url, Transfer::Type op_type, dav_ssize_t bytes_transfered, dav_size_t total_size, Transfer::Type tool_type){
+    static bool print_header= false;
+    static Chrono::TimePoint start_time, last_time;
+    Chrono::TimePoint current_time;
+    Chrono::Clock clk(Chrono::Clock::Monolitic, Chrono::Clock::Second);
+
+    int out_fd = STDERR_FILENO; // output fd -> stderr
+
+    if((op_type != tool_type)){
+        return;
+    }
+
+    int progress = 0;
+    dav_size_t baudrate =0;
+    current_time = clk.now();
+
+    if(print_header == false){
+        std::string type = ((op_type == Transfer::Read) ?  "Read" : "Write");
+        last_time = start_time = clk.now();
+
+        std::cout << "Performing " << type << " operation on: " << url << std::endl;
+        print_header = true;
+    }else{
+        // tty display is slow
+        // display progression only every second
+        if(current_time < (last_time + Chrono::Duration(1)) && (static_cast<dav_size_t>(bytes_transfered) != total_size))
+            return;
+    }
+    last_time = clk.now();
+    Chrono::Duration diff_time = last_time - start_time;
+    if( diff_time.toTimeValue() > 0){
+        baudrate = bytes_transfered/(diff_time.toTimeValue());
+    }
+
+
+    if(total_size > 0){
+        progress = static_cast<int>((static_cast<double>(bytes_transfered) / static_cast<double>(total_size)) * 100);
+    }
+
+    printProgressBar(out_fd, progress, bytes_transfered, total_size, baudrate);
+}
+
+
+std::string normalize_unit(dav_size_t bytes){
+    const char * tab_unit[] = { "", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei" };
+    size_t size_tab_unit= sizeof(tab_unit)/sizeof(const char*);
+
+    const dav_size_t factor = 10*1024;
+    size_t i;
+
+    for(i =0; i < size_tab_unit; ++i){
+        if(bytes < factor){
+            break;
+        }
+        bytes /= 1024;
+    }
+    return fmt::format("{}{}", bytes, tab_unit[i]);
+}
+
+void printProgressBar(int out_fd, int percent, dav_ssize_t bytes_transfered, dav_size_t total_size, dav_size_t baudrate){
+    std::string bar;
+    std::string unit;
+    static bool printOnce = true;
+    dav_size_t size = total_size;
+
+    struct winsize win;
+    ioctl(out_fd, TIOCGWINSZ, &win);
+
+    
+    // w = width of bar, r = 100/w
+    double w = static_cast<double>(win.ws_row)/1.5;
+    double r = 100/w;
+    for(double i = 0; i < w; ++i){
+        if(i < (static_cast<double>(percent)/r) ){
+            bar.replace(i,1,"=");
+        }else{
+            bar.replace(i,1," ");
+        }
+    }
+
+    if((percent == 100) && !printOnce)
+        return;
+
+    std::cout << "\r" "[" << bar << "] ";
+    std::cout.width(3);
+    std::cout << percent << "%     " << 
+        normalize_unit(bytes_transfered) << 'B' << "/" << normalize_unit(size) << 'B';
+    std::cout.width(10); 
+    std::cout << normalize_unit(baudrate) << "B/s" <<
+        "                     " << std::flush;
+
+    if(static_cast<dav_size_t>(bytes_transfered) == size){
+        std::cout << std::endl;
+        printOnce = false;
+    }
+}
+
+int configureMonitorCB(OptParams & opts, Transfer::Type type){
+    using namespace std;
+
+    // active monitoring callback only if output != stdout and stderr is a terminal
+    if(opts.output_file_path.empty() == false
+       && isatty(STDERR_FILENO) ==1 ){
+        TransferMonitorCB transMonCB;
+        transMonCB = bind(&Tool::TransferMonitor, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3, std::placeholders::_4, type);
+        opts.params.setTransfertMonitorCb(transMonCB);
+    }
+
+    return 0;
+}
+
+
 }
 }
