@@ -5,13 +5,4 @@
 #include <utils/davix_uri.hpp>
 #include <params/davixrequestparams.hpp>
 
-namespace Davix{
-
-void configureRequestParamsProto(const Uri & uri, RequestParams & params);
-
-
-
-
-}
-
 #endif // DAVIX_UTILS_INTERNAL_HPP
